@@ -8,28 +8,29 @@
 //               
 
 /*
-$Id: base_matrix.c,v 1.7 1995/12/20 16:35:47 lidiaadm Exp $
+$Id: base_matrix.c,v 1.15 1997/02/05 17:19:01 lidiaadm Exp $
 */
 
 #if defined(HAVE_MAC_DIRS) || defined(__MWERKS__)
 #include <LiDIA:base_matrix.h>
 #include <LiDIA:arith.inl>
 #include <LiDIA:xerror.h>
-#include <LiDIA:matrix_error_msg.c>
 #else
 #include <LiDIA/base_matrix.h>
 #include <LiDIA/arith.inl>
 #include <LiDIA/xerror.h>
-#include <LiDIA/matrix_error_msg.c>
 #endif
 
 /**
  ** debug defines / error defines
  **/
 
-#define DV_BM LDBL_MATRIX               /* Debug value   */
-#define DM_BM "base_matrix"             /* Debug message / Error message */
-#define ERROR matrix_error_msg
+extern const char *PRT;
+extern const char *matrix_error_msg[];
+
+#define DV_BM LDBL_MATRIX        /* Debug value   */
+#define DM_BM "base_matrix"      /* Debug message / Error message */
+#define LBM_ERROR matrix_error_msg
 
 /**
  ** debug level
@@ -55,7 +56,11 @@ base_matrix < T >::base_matrix()
   debug_handler_l(DM_BM, "in constructor "
 		  "base_matrix()", DV_BM);
   
-  print_mode = DEFAULT_MODE;
+  storage_mode = DEFAULT_STORAGE_MODE;
+  structure_mode = DEFAULT_STRUCTURE_MODE;
+  info_mode = DEFAULT_INFO_MODE;
+  print_mode = DEFAULT_PRINT_MODE;
+  lattice_mode = DEFAULT_LATTICE_MODE;
   rows = 1;
   columns = 1;
   
@@ -79,9 +84,13 @@ base_matrix < T >::base_matrix(lidia_size_t r)
     lidia_error_handler_para(r, "r", "r > 0", 
 			     "base_matrix < T >::"
 			     "base_matrix(lidia_size_t r)", 
-			     DM_BM, ERROR[0]);
+			     DM_BM, LBM_ERROR[0]);
   
-  print_mode = DEFAULT_MODE;
+  storage_mode = DEFAULT_STORAGE_MODE;
+  structure_mode = DEFAULT_STRUCTURE_MODE;
+  info_mode = DEFAULT_INFO_MODE;
+  print_mode = DEFAULT_PRINT_MODE;
+  lattice_mode = DEFAULT_LATTICE_MODE;
   rows = r;
   columns = 1;
   
@@ -110,9 +119,13 @@ base_matrix < T >::base_matrix(lidia_size_t r, lidia_size_t c)
 			     c, "c", "c > 0", 
 			     "base_matrix < T >::"
 			     "base_matrix(lidia_size_t r, lidia_size_t c)",
-			     DM_BM, ERROR[0]);
+			     DM_BM, LBM_ERROR[0]);
   
-  print_mode = DEFAULT_MODE;
+  storage_mode = DEFAULT_STORAGE_MODE;
+  structure_mode = DEFAULT_STRUCTURE_MODE;
+  info_mode = DEFAULT_INFO_MODE;
+  print_mode = DEFAULT_PRINT_MODE;
+  lattice_mode = DEFAULT_LATTICE_MODE;
   rows = r;
   columns = c;
   
@@ -130,34 +143,39 @@ base_matrix < T >::base_matrix(lidia_size_t r, lidia_size_t c)
 }
 
 template < class T >
-base_matrix < T >::base_matrix(lidia_size_t r, lidia_size_t c, T **A)
+base_matrix < T >::base_matrix(lidia_size_t r, lidia_size_t c, const T **A)
 {
   debug_handler_l(DM_BM, "in constructor "
-		  "base_matrix(lidia_size_t, lidia_size_t, T **)", DV_BM);
+		  "base_matrix(lidia_size_t, lidia_size_t, const T **)", DV_BM);
   
   if (r < 1 || c < 1 || A == NULL)
     lidia_error_handler_para(r, "r", "r > 0", 
 			     c, "c", "c > 0", 
 			     PRT, "A", "A != NULL",
 			     "base_matrix < T >::"
-			     "base_matrix(lidia_size_t r, lidia_size_t c, T **A)", 
-			     DM_BM, ERROR[1]);
+			     "base_matrix(lidia_size_t r, lidia_size_t c, const T **A)", 
+			     DM_BM, LBM_ERROR[1]);
   
-  print_mode = DEFAULT_MODE;
+  storage_mode = DEFAULT_STORAGE_MODE;
+  structure_mode = DEFAULT_STRUCTURE_MODE;
+  info_mode = DEFAULT_INFO_MODE;
+  print_mode = DEFAULT_PRINT_MODE;
+  lattice_mode = DEFAULT_LATTICE_MODE;
   rows = r;
   columns = c;
   
   value = new T *[r];
-  memory_handler(value, DM_BM, "base_matrix(lidia_size_t, lidia_size_t, T **) :: "
+  memory_handler(value, DM_BM, "base_matrix(lidia_size_t, lidia_size_t, const T **) :: "
 		 "Error in memory allocation (value)");
 
   register lidia_size_t i, j;
-  register T *tmp, *Atmp;
+  register T *tmp;
+  register const T *Atmp;
   for (i = 0; i < r; i++)
     {
       tmp = new T[c];
       Atmp = A[i];
-      memory_handler(tmp, DM_BM, "base_matrix(lidia_size_t, lidia_size_t, T **) :: "
+      memory_handler(tmp, DM_BM, "base_matrix(lidia_size_t, lidia_size_t, const T **) :: "
 		     "Error in memory allocation (tmp)");
       
       for (j = 0; j < c; j++)
@@ -172,7 +190,11 @@ base_matrix < T >::base_matrix(const base_matrix < T > &M)
   debug_handler_l(DM_BM, "in constructor "
 		  "base_matrix(const base_matrix < T > &)", DV_BM);
   
+  storage_mode = M.storage_mode;
+  structure_mode = M.structure_mode;
+  info_mode = M.info_mode;
   print_mode = M.print_mode;
+  lattice_mode = M.lattice_mode;
   rows = M.rows;
   columns = M.columns;
   
@@ -319,7 +341,7 @@ read(istream &in)
       lidia_error_handler_para(c, "c", "Character c have to be in {[,{,a,M,1,2,3,4,5,6,7,8,9}",
 			       "istream & base_matrix < T >::"
 			       "read(istream &in)", 
-			       DM_BM, ERROR[2]);
+			       DM_BM, LBM_ERROR[2]);
     }
   return in;
 }
@@ -355,13 +377,41 @@ sto(lidia_size_t x, lidia_size_t y, const T &e)
 			     columns, "columns", "0 <= y < columns",
 			     "void base_matrix < T >::"
 			     "sto(lidia_size_t x, lidia_size_t y, const T &e)", 
-			     DM_BM, ERROR[3]);  
+			     DM_BM, LBM_ERROR[3]);  
   
   value[x][y] = e;
 }
 
 template < class T >
-T base_matrix < T >::
+T & base_matrix < T >::
+operator () (lidia_size_t x, lidia_size_t y)
+{
+  /**
+   ** DESCRIPTION: A(x,y) = reference of position (x,y) in matrix A
+   ** ERROR: x < 0 or 
+   **        y < 0 or 
+   **        x >= A.rows or 
+   **        y >= A.columns
+   ** VERSION: 2.0
+   **/
+  
+  debug_handler_l(DM_BM, "operator()"
+		  "(lidia_size_t, lidia_size_t)", DV_BM + 2);
+  
+  if (x < 0 || y < 0 || x >= rows || y >= columns)
+    lidia_error_handler_para(x, "x", "0 <= x < rows",
+			     rows, "rows", "0 <= x < rows",
+			     y, "y", "0 <= y < columns",
+			     columns, "columns", "0 <= y < columns",
+			     "T base_matrix < T >::"
+			     "operator () (lidia_size_t x, lidia_size_t y) const", 
+			     DM_BM, LBM_ERROR[3]);
+  
+  return value[x][y];
+}
+
+template < class T >
+const T & base_matrix < T >::
 member(lidia_size_t x, lidia_size_t y) const
 {
   /**
@@ -383,7 +433,7 @@ member(lidia_size_t x, lidia_size_t y) const
 			     columns, "columns", "0 <= y < columns",
 			     "T base_matrix < T >::"
 			     "member(lidia_size_t x, lidia_size_t y) const", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   return value[x][y];
 }
@@ -418,7 +468,7 @@ sto_column(const T *v, lidia_size_t l, lidia_size_t j)
 			     PRT, "v", " v != NULL",
 			     "void base_matrix < T >::"
 			     "sto_column(const T *v, lidia_size_t l, lidia_size_t j)",
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   register lidia_size_t k;
 
@@ -452,7 +502,7 @@ sto_column_vector(const base_vector < T > &v, lidia_size_t l, lidia_size_t j)
 			     v.size(), "v.size", "0 < l <= min(v.size, rows)",
 			     "void base_matrix < T >::"
 			     "sto_column_vector(const base_vector < T > &v, lidia_size_t l, lidia_size_t j)",
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   register T *tmpv = v.get_data_address(); 
   
@@ -482,7 +532,7 @@ column(lidia_size_t i) const
 			     columns, "columns", "0 <= i < columns",
 			     "T *base_matrix < T >::"
 			     "column(lidia_size_t i) const", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   register T *RES = new T[rows];
   memory_handler(RES, DM_BM, "column :: "
@@ -518,7 +568,7 @@ column(T *RES, lidia_size_t i) const
 			     PRT, "RES", "RES != NULL",
 			     "void base_matrix < T >::"
 			     "column(T *RES, lidia_size_t i) const", 
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
 
   register lidia_size_t k;
 
@@ -547,7 +597,7 @@ column_vector(base_vector < T > &RES, lidia_size_t i) const
 			     columns, "columns", "0 <= i < columns",
 			     "void base_matrix < T >::"
 			     "column_vector(base_vector < T > &RES, lidia_size_t i) const", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   if (RES.capacity() < rows)
     RES.set_capacity(rows);
@@ -590,7 +640,7 @@ sto_row(const T *v, lidia_size_t l, lidia_size_t i)
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "sto_row(const T *v, lidia_size_t l, lidia_size_t i)", 
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   register T *tmp = value[i];
   register lidia_size_t k;
@@ -624,7 +674,7 @@ sto_row_vector(const base_vector < T > &v, lidia_size_t l, lidia_size_t i)
 			     v.size(), "v.size", "0 < l <= min(columns, v.size)",
 			     "void base_matrix < T >::"
 			     "sto_row_vector(const base_vector < T > &v, lidia_size_t l, lidia_size_t i)", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   register T *tmp = value[i];
   register T *tmpv = v.get_data_address(); 
@@ -655,7 +705,7 @@ row(lidia_size_t i) const
 			     rows, "rows", "0 <= i < rows",
 			     "T *base_matrix < T >::"
 			     "row(lidia_size_t i) const", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   register T *RES = new T[columns];
   memory_handler(RES, DM_BM, "row :: "
@@ -692,7 +742,7 @@ row(T *RES, lidia_size_t i) const
 			     PRT, "RES", "RES != NULL", 
 			     "void base_matrix < T >::"
 			     "row(T *RES, lidia_size_t i) const", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   register lidia_size_t k;
 
@@ -721,7 +771,7 @@ row_vector(base_vector < T > &RES, lidia_size_t i) const
 			     rows, "rows", "0 <= i < rows",
 			     "void base_matrix < T >::"
 			     "row_vector(base_vector < T > &RES, lidia_size_t i) const", 
-			     DM_BM, ERROR[3]);
+			     DM_BM, LBM_ERROR[3]);
   
   if (RES.capacity() < columns)
     RES.set_capacity(columns);
@@ -791,7 +841,7 @@ set_data(const T **v, lidia_size_t r, lidia_size_t c)
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "set_data(const T **v, lidia_size_t r, lidia_size_t c)", 
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   if (r != rows)
     set_no_of_rows(r);
@@ -861,7 +911,7 @@ split_t(base_matrix < T > &A, base_matrix < T > &B, base_matrix < T > &C, base_m
 			     "void base_matrix < T >::"
 			     "split(base_matrix < T > &A, base_matrix < T > &B, "
 			     "base_matrix < T > &C, base_matrix < T > &D) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   register lidia_size_t i, j;
   register T *tmp, *tmp1;
@@ -894,7 +944,7 @@ split_t(base_matrix < T > &A, base_matrix < T > &B, base_matrix < T > &C, base_m
     {
       tmp = &value[rows - D.rows + i][columns - D.columns];
       tmp1 = D.value[i];
-      for (j = 0; j < B.columns; j++)
+      for (j = 0; j < D.columns; j++)
 	tmp1[j] = tmp[j];
     }
 }
@@ -927,7 +977,7 @@ split_h(base_matrix < T > &A, base_matrix < T > &B) const
 			     columns, "columns", "",
 			     "void base_matrix < T >::"
 			     "split_h(base_matrix < T > &A, base_matrix < T > &B) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   register T *tmp, *tmp1;  
@@ -974,7 +1024,7 @@ split_h(T *v, base_matrix < T > &A) const
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "split_h(T *v, base_matrix < T > &A) const",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   register lidia_size_t i, j;
   register T *tmp, *tmp1;  
@@ -1019,7 +1069,7 @@ split_h(base_vector < T > &v, base_matrix < T > &A) const
 			     l, "v.size", "v.size <= rows",
 			     "void base_matrix < T >::"
 			     "split_h(base_vector < T > &v, base_matrix < T > &A) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register T *tmp = v.get_data_address();
 
@@ -1061,7 +1111,7 @@ split_h(base_matrix < T > &A, T *v) const
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "split_h(base_matrix < T > &A, T *v) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register T *tmp, *tmp1;  
   register lidia_size_t i, j;  
@@ -1107,7 +1157,7 @@ split_h(base_matrix < T > &A, base_vector < T > &v) const
 			     l, "v.size", "v.size <= rows",
 			     "void base_matrix < T >::"
 			     "split_h(base_matrix < T > &A, base_vector < T > &v) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;  
   for (i = 0; i < A.rows; i++)
@@ -1154,7 +1204,7 @@ split_v(base_matrix < T > &A, base_matrix < T > &B) const
 			     columns, "columns", "",
 			     "void base_matrix < T >::"
 			     "split_v(base_matrix < T > &A, base_matrix < T > &B) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   register T *tmp, *tmp1;  
@@ -1202,7 +1252,7 @@ split_v(T *v, base_matrix < T > &A) const
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "split_v(T *v, base_matrix < T > &A) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   register T *tmp1, *tmp = value[0];
@@ -1247,7 +1297,7 @@ split_v(base_vector < T > &v, base_matrix < T > &A) const
 			     l, "v.size", "v.size <= columns",
 			     "void base_matrix < T >::"
 			     "split_v(base_vector < T > &v, base_matrix < T > &A) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   register T *tmp = value[0];
   register T *tmp1 = v.get_data_address();
@@ -1294,7 +1344,7 @@ split_v(base_matrix < T > &A, T *v) const
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "split_v(base_matrix < T > &A, T *v) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   for (i = 0; i < A.rows; i++)
     {
@@ -1338,7 +1388,7 @@ split_v(base_matrix < T > &A, base_vector < T > &v) const
 			     l, "v.size", "v.size <= columns",
 			     "void base_matrix < T >::"
 			     "split_v(base_matrix < T > &A, base_vector < T > &v) const", 
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;   
 
@@ -1401,7 +1451,7 @@ compose_t(const base_matrix < T > &A, const base_matrix < T > &B, const base_mat
 			     "void base_matrix < T >::"
 			     "compose(const base_matrix < T > &A, const base_matrix < T > &B, "
 			     "const base_matrix < T > &C, const base_matrix < T > &D)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   register lidia_size_t i, j;
   T *tmp, *tmp1;  
@@ -1466,7 +1516,7 @@ compose_h(const base_matrix < T > &A, const base_matrix < T > &B)
 			     columns, "columns", "",
 			     "void base_matrix < T >::"
 			     "compose_h(const base_matrix < T > &A, const base_matrix < T > &B)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   register lidia_size_t i, j;
   T *tmp, *tmp1;
@@ -1512,7 +1562,7 @@ compose_h(const T *v, const base_matrix < T > &A)
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "compose_h(const T *v, const base_matrix < T > &A)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   T *tmp, *tmp1;  
@@ -1556,7 +1606,7 @@ compose_h(const base_vector < T > &v, const base_matrix < T > &A)
 			     l, "v.size", "v.size <= rows",
 			     "void base_matrix < T >::"
 			     "compose_h(const base_vector < T > &v, const base_matrix < T > &A)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   T *tmp = v.get_data_address();
 
@@ -1598,7 +1648,7 @@ compose_h(const base_matrix < T > &A, const T *v)
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "compose_h(const base_matrix < T > &A, const T *v)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   T *tmp, *tmp1;  
@@ -1643,7 +1693,7 @@ compose_h(const base_matrix < T > &A, const base_vector < T > &v)
 			     l, "v.size", "v.size <= rows",
 			     "void base_matrix < T >::"
 			     "compose_h(const base_matrix < T > &A, const base_vector < T > &v)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;  
   for (i = 0; i < A.rows; i++)
@@ -1689,7 +1739,7 @@ compose_v(const base_matrix < T > &A, const base_matrix < T > &B)
 			     columns, "columns", "",
 			     "void base_matrix < T >::"
 			     "compose_v(const base_matrix < T > &A, const base_matrix < T > &B)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   T *tmp, *tmp1;  
@@ -1736,7 +1786,7 @@ compose_v(const T *v, const base_matrix < T > &A)
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "compose_v(const T *v, const base_matrix < T > &A)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   T *tmp1;  
@@ -1781,7 +1831,7 @@ compose_v(const base_vector < T > &v, const base_matrix < T > &A)
 			     l, "v.size", "v.size <= coluns",
 			     "void base_matrix < T >::"
 			     "compose_v(const base_vector < T > &v, const base_matrix < T > &A)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
   
   T *tmp = value[0];
   T *tmp1 = v.get_data_address();
@@ -1825,7 +1875,7 @@ compose_v(const base_matrix < T > &A, const T *v)
 			     PRT, "v", "v != NULL",
 			     "void base_matrix < T >::"
 			     "compose_v(const base_matrix < T > &A, const T *v)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;
   T *tmp, *tmp1;
@@ -1871,7 +1921,7 @@ compose_v(const base_matrix < T > &A, const base_vector < T > &v)
 			     l, "v.size", "v.size <= columns",
 			     "void base_matrix < T >::"
 			     "compose_v(const base_matrix < T > &A, const base_vector < T > &v)",
-			     DM_BM, ERROR[4]);
+			     DM_BM, LBM_ERROR[4]);
 
   register lidia_size_t i, j;   
   for (i = 0; i < A.rows; i++)
@@ -1941,7 +1991,7 @@ swap_columns(lidia_size_t i, lidia_size_t j)
 			     columns, "columns", "",
 			     "void base_matrix < T >::"
 			     "swap_columns(lidia_size_t i, lidia_size_t j)",
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   register lidia_size_t z;
   T *tmp;
@@ -1975,7 +2025,7 @@ swap_rows(lidia_size_t i, lidia_size_t j)
 			     rows, "rows", "",
 			     "void base_matrix < T >::"
 			     "swap_rows(lidia_size_t i, lidia_size_t j)",
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
 
   T *tmp = value[i];
   value[i] = value[j];
@@ -2003,7 +2053,7 @@ set_no_of_rows(lidia_size_t r)
     lidia_error_handler_para(r, "r", "r > 0", 
 			     "void base_matrix < T >::"
 			     "set_no_of_rows(lidia_size_t r)",
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   if (r == rows)
     return;
@@ -2055,7 +2105,7 @@ set_no_of_columns(lidia_size_t c)
     lidia_error_handler_para(c, "c", "c > 0", 
 			     "void base_matrix < T >::"
 			     "set_no_of_columns(lidia_size_t c)",
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   if (columns == c)
     return;
@@ -2114,7 +2164,7 @@ resize(lidia_size_t r, lidia_size_t c)
 			     c, "c", "c > 0",
 			     "void base_matrix < T >::"
 			     "resize(lidia_size_t r, lidia_size_t c)",
-			     DM_BM, ERROR[1]);
+			     DM_BM, LBM_ERROR[1]);
   
   if (columns != c)
     set_no_of_columns(c);
@@ -2150,6 +2200,8 @@ kill()
       value[0] = new T[1];
       memory_handler(value[0], DM_BM, "kill :: "
 		     "Error in memory allocation (value[0])");
+      rows = 1;
+      columns =1;
     }
 }
  
@@ -2186,6 +2238,11 @@ assign(const base_matrix < T > &M)
       for (j = 0; j < columns; j++)
 	tmp[j] = Mtmp[j];
     }
+  storage_mode = M.storage_mode;
+  structure_mode = M.structure_mode;
+  info_mode = M.info_mode;
+  print_mode = M.print_mode;
+  lattice_mode = M.lattice_mode;
 }
 
 /**
@@ -2430,7 +2487,7 @@ read_from_mathematica(istream &dz)
   if (c != '{')
     lidia_error_handler_para("void base_matrix < T >::"
 			     "read_from_mathematica(istream &dz)",
-			     DM_BM, ERROR[5]);
+			     DM_BM, LBM_ERROR[5]);
   
   register lidia_size_t i1 = 0, i2 = 0, i3 = 1, i, j;
   
@@ -2520,7 +2577,7 @@ read_from_maple(istream &dz)
   if (dz.bad())
     lidia_error_handler_para("void base_matrix < T >::"
 			     "read_from_maple(istream &dz)", 
-			     DM_BM, ERROR[5]);
+			     DM_BM, LBM_ERROR[5]);
   
   register lidia_size_t i, j;
   lidia_size_t startr, startc, endr, endc, x, y;
@@ -2613,7 +2670,7 @@ read_from_gp(istream &dz)
   if (c != '[')
     lidia_error_handler_para("void base_matrix < T >::"
 			     "read_from_gp(istream &dz)",
-			     DM_BM, ERROR[5]);
+			     DM_BM, LBM_ERROR[5]);
   
   register lidia_size_t i, j, i1 = 0, i2 = 0, i3 = 1;  
   while (!dz.eof() && i3 != 0)
@@ -2704,7 +2761,7 @@ read_from_kash(istream &dz)
   if (c != 'M')
     lidia_error_handler_para("void base_matrix < T >::"
 			     "read_from_kash(istream &dz)",
-			     DM_BM, ERROR[5]);
+			     DM_BM, LBM_ERROR[5]);
 
   dz >> ws >> c >> ws >> c >> ws >> c >> ws >> c >> ws >> c >> ws >> c;
 
@@ -2745,7 +2802,9 @@ read_from_kash(istream &dz)
     }
 }
 
-
+#undef DV_BM 
+#undef DM_BM 
+#undef LBM_ERROR
 
 
 
