@@ -30,21 +30,22 @@
 #include <LiDIA:base_vector.h>
 #include <LiDIA:arith.inl>
 #include <LiDIA:xerror.h>
-#include <LiDIA:vector_error_msg.c>
 #else
 #include <LiDIA/base_vector.h>
 #include <LiDIA/arith.inl>
 #include <LiDIA/xerror.h>
-#include <LiDIA/vector_error_msg.c>
 #endif
 
 /**
  ** debug defines / error defines
  **/
 
-#define DV_BV LDBL_VECTOR               /* Debug value   */
-#define DM_BV "base_vector"             /* Debug message / Error message */
-#define ERROR vector_error_msg
+extern const char *PRT;
+extern const char *vector_error_msg[];
+
+#define DV_BV LDBL_VECTOR         /* Debug value   */
+#define DM_BV "base_vector"       /* Debug message / Error message */
+#define LBV_ERROR vector_error_msg
 
 /**
  ** debug level
@@ -58,6 +59,7 @@
  **   6 : assignments
  **   7 : reverse functions
  **   8 : stream handling
+ **   9 : remove / insert
  **/
 
 template < class T >
@@ -67,9 +69,7 @@ copy_data(T *d, const T *vd, lidia_size_t l)
   debug_handler_l(DM_BV, "in member - function "
 		  "copy_data(T *, T *, lidia_size_t)", DV_BV);
   
-  register lidia_size_t i;
-
-  for (i = 0; i < l; i++)
+  for (register lidia_size_t i = 0; i < l; i++)
     d[i] = vd[i];
   return d;
 }	
@@ -128,7 +128,7 @@ base_vector < T >::base_vector(lidia_size_t all)
       lidia_error_handler_para(all, "all", "all > 0",
 			       "base_vector < T >::"
 			       "base_vector(lidia_size_t all)",
-			       DM_BV, ERROR[0]);
+			       DM_BV, LBV_ERROR[0]);
 }
 */
 
@@ -155,7 +155,7 @@ base_vector < T >::base_vector(lidia_size_t all, char md)
       lidia_error_handler_para(all, "all", "all > 0",
 			       "base_vector < T >::"
 			       "base_vector(lidia_size_t all, char md)",
-			       DM_BV, ERROR[0]);
+			       DM_BV, LBV_ERROR[0]);
 }
 
 template < class T >
@@ -181,7 +181,7 @@ base_vector < T >::base_vector(lidia_size_t all, lidia_size_t len)
       lidia_error_handler_para(all, "all", "all > 0",
 			       "base_vector < T >::"
 			       "base_vector(lidia_size_t all, lidia_size_t len)",
-			       DM_BV, ERROR[0]);
+			       DM_BV, LBV_ERROR[0]);
 }
 
 template < class T >
@@ -207,7 +207,7 @@ base_vector < T >::base_vector(lidia_size_t all, lidia_size_t len, char md)
       lidia_error_handler_para(all, "all", "all > 0",
 			       "base_vector < T >::"
 			       "base_vector(lidia_size_t all, lidia_size_t len, char md)",
-			       DM_BV, ERROR[0]);
+			       DM_BV, LBV_ERROR[0]);
 }
 
 template < class T >
@@ -281,7 +281,7 @@ base_vector < T >::base_vector(const T *v, lidia_size_t len)
       lidia_error_handler_para(allocated, "allocated", "allocated > 0",
 			       "base_vector < T >::"
 			       "base_vector(const T *v, lidia_size_t len)",
-			       DM_BV, ERROR[6]);
+			       DM_BV, LBV_ERROR[6]);
 }
 
 template < class T >
@@ -309,7 +309,7 @@ base_vector < T >::base_vector(const T *v, lidia_size_t len, char md)
       lidia_error_handler_para(allocated, "allocated", "allocated > 0",
 			       "base_vector < T >::"
 			       "base_vector(const T *v, lidia_size_t len, char md)",
-			       DM_BV, ERROR[6]);
+			       DM_BV, LBV_ERROR[6]);
 }
 
 /**
@@ -470,7 +470,7 @@ set_capacity(lidia_size_t all)
 	lidia_error_handler_para(all, "all", "all > 0",
 				 "void base_vector < T >::"
 				 "set_capacity(lidia_size_t all)",
-				 DM_BV, ERROR[0]);
+				 DM_BV, LBV_ERROR[0]);
 }
 
 template < class T >
@@ -501,7 +501,7 @@ set_size(lidia_size_t len)
     lidia_error_handler_para(len, "len", "len > 0",
 			     "void base_vector < T >::"
 			     "set_size(lidia_size_t len)",
-			     DM_BV, ERROR[0]);
+			     DM_BV, LBV_ERROR[0]);
   else 
     if (len <= allocated)
       length = len ;
@@ -516,39 +516,7 @@ set_size(lidia_size_t len)
 	lidia_error_handler_para(mode, "mode", "mode == EXPAND",
 				 "void base_vector < T >::"
 				 "set_size(lidia_size_t len)", 
-				 DM_BV, ERROR[7]);
-}
-
-template < class T >
-void base_vector < T >::
-set_exp_ratio(float ratio)
-{
-  debug_handler_l("dyn_vector", "in member - function "
-		  "set_exp_ratio(float)", DV_BV + 5);
-  
-  if (ratio >= 1)
-    dyn_exp_ratio = ratio;
-  else
-    lidia_error_handler_para(ratio, "ratio", "ratio >= 1",
-			     "void base_vector < T >::"
-			     "set_exp_ratio(float ratio)",
-			     DM_BV, ERROR[8]);
-}
-
-template < class T >
-void base_vector < T >::
-set_mode(char md)
-{
-  debug_handler_l(DM_BV, "in member - function "
-		  "set_mode(char)", DV_BV + 5);
-  
-  if (md == FIXED || md == EXPAND)
-    mode = md;
-  else
-    lidia_error_handler_para(md, "md", "md == FIXED od md == EXPAND",
-			     "void base_vector < T >::"
-			     "set_mode(char md)",
-			     DM_BV, ERROR[9]);
+				 DM_BV, LBV_ERROR[7]);
 }
 
 /**
@@ -566,7 +534,7 @@ operator [](lidia_size_t i)
     lidia_error_handler_para(i, "i", "i >= 0", 
 			     "T & base_vector < T >::"
 			     "operator [](lidia_size_t i)",
-			     DM_BV, ERROR[3]);
+			     DM_BV, LBV_ERROR[3]);
   else
     if (i < allocated)
       length = (i + 1 > length) ? ( i + 1 ) : length;
@@ -581,7 +549,7 @@ operator [](lidia_size_t i)
 				 mode, "mode", "mode == EXPAND",
 				 "T & base_vector < T >::"
 				 "operator [](lidia_size_t i)",
-				 DM_BV, ERROR[3]);
+				 DM_BV, LBV_ERROR[3]);
   return data[i];
 }
 
@@ -596,12 +564,12 @@ operator [](lidia_size_t i) const
     lidia_error_handler_para(i, "i", "0 <= i < length",
 			     "const T & base_vector < T >::"
 			     "operator [](lidia_size_t i) const",
-			     DM_BV, ERROR[3]);
+			     DM_BV, LBV_ERROR[3]);
   return data[i];      
 }
 
 template < class T >
-T & base_vector < T >::
+const T & base_vector < T >::
 member(lidia_size_t i) const
 {
   debug_handler_l(DM_BV, "in meber - function "
@@ -611,7 +579,7 @@ member(lidia_size_t i) const
     lidia_error_handler_para(i, "i", "0 <= i < length",
 			     "T & base_vector < T >::"
 			     "member(lidia_size_t i) const",
-			     DM_BV, ERROR[3]);
+			     DM_BV, LBV_ERROR[3]);
   return data[i];      
 }
 
@@ -662,6 +630,45 @@ operator = (const base_vector < T > & v)
   return *this;
 }
 
+template <class T >
+void base_vector < T >::
+assign(lidia_size_t at, const base_vector< T > & v, lidia_size_t from, lidia_size_t to)
+{
+  debug_handler_l(DM_BV, "in member - function "
+		  "assign(lidia_size_t, base_vector, lidia_size_t, lidia_size_t)", DV_BV + 3);
+  
+  register lidia_size_t i, j;
+  
+  if (at < 0 || from < 0 || from >= v.length || to < 0 || to >= v.length || to < from)
+    lidia_error_handler_para(DM_BV, "assign :: "
+			"invalid indices");
+  
+  register lidia_size_t n = to - from + 1;  /* this value is positive */
+  register lidia_size_t new_len = (length < at + n) ? ( at + n ) : length;
+  
+  if (this != &v)                    /* v aliases current instance */
+    {
+      set_size(new_len);
+      for (i = from, j = at; i <= to; i++, j++)
+	data[j] = v.data[i];
+    }
+  else
+    {
+      set_size(new_len);
+      T *tmp = new T[n];
+      memory_handler(tmp, DM_BV, "assign :: "
+		     "out of memory");
+	  
+      for (i = from, j = 0; i <= to; i++, j++)
+	tmp[j] = v.data[i];
+	  
+      for (i = 0, j = at; i < n; i++, j++)
+	data[j] = tmp[i];
+	  
+      delete[] tmp;
+    }
+}
+
 template < class T >
 void base_vector < T >::
 assign(const base_vector < T > &v)  
@@ -754,7 +761,7 @@ swap(base_vector < T > & b)
 
 
 /**
- ** concat / divide / shift / assign
+ ** concat function
  **/
 
 template < class T >
@@ -815,6 +822,10 @@ concat(const base_vector < T > &b, const base_vector < T > &c)
   allocated = length = b.length + c.length;
 }
 
+/**
+ ** shift functions
+ **/
+
 template < class T >
 void base_vector < T >::
 shift_left(lidia_size_t pos, lidia_size_t num)
@@ -829,7 +840,7 @@ shift_left(lidia_size_t pos, lidia_size_t num)
     lidia_error_handler_para (pos, "pos", "pos >= num",
 			      "void base_vector < T >::"
 			      "shift_left(lidia_size_t pos, lidia_size_t num)",
-			      DM_BV, ERROR[3]);
+			      DM_BV, LBV_ERROR[3]);
   else
     {
       set_size(old_len - num);
@@ -852,7 +863,7 @@ shift_right(lidia_size_t pos, lidia_size_t num)
     lidia_error_handler_para (pos, "pos", "pos >= 0",
 			      "void base_vector < T >::"
 			      "shift_right(lidia_size_t pos, lidia_size_t num)",
-			      DM_BV, ERROR[3]);
+			      DM_BV, LBV_ERROR[3]);
   else
     {
       set_size(old_len + num);
@@ -861,51 +872,16 @@ shift_right(lidia_size_t pos, lidia_size_t num)
     }
 } 
 
-template <class T >
-void base_vector < T >::
-assign(lidia_size_t at, const base_vector< T > & v, lidia_size_t from, lidia_size_t to)
-{
-  debug_handler_l(DM_BV, "in member - function "
-		  "assign(lidia_size_t, base_vector, lidia_size_t, lidia_size_t)", DV_BV + 3);
-  
-  register lidia_size_t i, j;
-  
-  if (at < 0 || from < 0 || from >= v.length || to < 0 || to >= v.length || to < from)
-    lidia_error_handler_para(DM_BV, "assign :: "
-			"invalid indices");
-  
-  register lidia_size_t n = to - from + 1;  /* this value is positive */
-  register lidia_size_t new_len = (length < at + n) ? ( at + n ) : length;
-  
-  if (this != &v)                    /* v aliases current instance */
-    {
-      set_size(new_len);
-      for (i = from, j = at; i <= to; i++, j++)
-	data[j] = v.data[i];
-    }
-  else
-    {
-      set_size(new_len);
-      T *tmp = new T[n];
-      memory_handler(tmp, DM_BV, "assign :: "
-		     "out of memory");
-	  
-      for (i = from, j = 0; i <= to; i++, j++)
-	tmp[j] = v.data[i];
-	  
-      for (i = 0, j = at; i < n; i++, j++)
-	data[j] = tmp[i];
-	  
-      delete[] tmp;
-    }
-}
+/**
+ ** remove function
+ **/
 
 template < class T >
 void base_vector < T >:: 
 remove_from(lidia_size_t pos, lidia_size_t l)
 {
   debug_handler_l(DM_BV, "in member - function "
-		  "remove_from(lidia_size_t, lidia_size_t)", DV_BV + 4);
+		  "remove_from(lidia_size_t, lidia_size_t)", DV_BV + 9);
   
   register lidia_size_t i;
   
@@ -914,9 +890,48 @@ remove_from(lidia_size_t pos, lidia_size_t l)
 			      pos, "pos", "0 <= pos and pos + l <= length",
 			      "void sort_vector < T >::"
 			      "remove_from(lidia_size_t pos, lidia_size_t l)",
-			      DM_BV, ERROR[12]);
+			      DM_BV, LBV_ERROR[12]);
   
   for (i = pos; i < length - l; i++)
     data[i] = data[i + l];
   set_size(length - l);
 }
+
+/**
+ ** insert function
+ **/
+
+template < class T >
+void base_vector < T >::
+insert_at(const T & x, lidia_size_t pos)
+{
+  debug_handler_l(DM_SV, "in member - function "
+		  "insert_at (const T &, lidia_size_t)", DV_BV + 9);
+  
+  register lidia_size_t i, l = length;
+  
+  if (pos < 0)
+    lidia_error_handler_para(pos, "pos", "pos >= 0",
+			     "void base_vector < T >::"
+			     "insert_at(const T & x, lidia_size_t pos)",
+			     DM_BV , LBV_ERROR[1]);
+           
+  set_size((l+1) > (pos + 1) ? l + 1 : pos + 1);
+  
+  /* if size has been increased, insert at position 'pos' */
+  for (i = length - 1; i > pos; i--)
+    data[i] = data[i - 1];
+  data[pos] = x;
+}
+
+#undef DV_BV
+#undef DM_BV 
+#undef LBV_ERROR
+
+
+
+
+
+
+
+
