@@ -89,9 +89,9 @@ class bigfloat_lattice_gensys:public math_matrix< bigfloat >
 // Constructor / Destructors 
 //
     bigfloat_lattice_gensys(); 
-    bigfloat_lattice_gensys(lidia_size_t);
+//    bigfloat_lattice_gensys(lidia_size_t);
     bigfloat_lattice_gensys(lidia_size_t, lidia_size_t);
-    bigfloat_lattice_gensys(lidia_size_t, lidia_size_t, bigfloat**);
+    bigfloat_lattice_gensys(lidia_size_t, lidia_size_t, const bigfloat**);
     bigfloat_lattice_gensys(const math_matrix < bigfloat >&);
     bigfloat_lattice_gensys(const bigfloat_lattice_gensys&);
     ~bigfloat_lattice_gensys();    
