@@ -100,9 +100,9 @@ class bigint_lattice_gensys : public math_matrix < bigint >
 // Constructor / Destructors 
 //
     bigint_lattice_gensys(); 
-    bigint_lattice_gensys(lidia_size_t);
+//    bigint_lattice_gensys(lidia_size_t);
     bigint_lattice_gensys(lidia_size_t, lidia_size_t);
-    bigint_lattice_gensys(lidia_size_t, lidia_size_t, bigint**);
+    bigint_lattice_gensys(lidia_size_t, lidia_size_t, const bigint**);
     bigint_lattice_gensys(const math_matrix < bigint >&);
     bigint_lattice_gensys(const bigint_lattice_gensys&);
     ~bigint_lattice_gensys();    
