@@ -8,7 +8,7 @@
 //
 
 /*
-$Id: bigint_matrix.h,v 1.19 1996/03/11 13:37:50 theobald Exp $
+$Id: bigint_matrix.h,v 1.22 1996/10/29 15:42:19 theobald Exp $
 */
 
 #ifndef LIDIA_BIGINT_MATRIX_H
@@ -44,7 +44,7 @@ class bigint_matrix : public math_matrix < bigint >
   
   inline bigint_matrix() : math_matrix < bigint > () {}
   inline bigint_matrix(lidia_size_t a, lidia_size_t b) : math_matrix < bigint > (a,b) {} 
-  inline bigint_matrix(lidia_size_t a, lidia_size_t b, bigint **v) : math_matrix < bigint > (a,b,v) {}
+  inline bigint_matrix(lidia_size_t a, lidia_size_t b, const bigint **v) : math_matrix < bigint > (a,b,v) {}
   inline bigint_matrix(const base_matrix < bigint > &A) : math_matrix < bigint > (A) {}
     
   /**
@@ -82,8 +82,8 @@ class bigint_matrix : public math_matrix < bigint >
    **/
 
  public:
-  
-  inline bigint_matrix & operator = (const base_matrix < bigint > &B)
+ 
+  inline bigint_matrix & operator = (const base_matrix < bigint >& B) 
     {base_matrix < bigint >::assign(B); return *this;}
   
   /**
