@@ -8,28 +8,29 @@
 //
 
 /*
-$Id: math_matrix.c,v 1.6 1996/04/12 17:32:31 lidiaadm Exp $
+$Id: math_matrix.c,v 1.10 1996/12/18 15:11:13 theobald Exp $
 */
 
 #if defined(HAVE_MAC_DIRS) || defined(__MWERKS__)
 #include <LiDIA:math_matrix.h>
 #include <LiDIA:arith.inl>
 #include <LiDIA:xerror.h>
-#include <LiDIA:matrix_error_msg.c>
 #else
 #include <LiDIA/math_matrix.h>
 #include <LiDIA/arith.inl>
 #include <LiDIA/xerror.h>
-#include <LiDIA/matrix_error_msg.c>
 #endif
 
 /**
  ** debug defines / error defines
  **/
 
-#define DV_MM LDBL_MATRIX + 10              /* Debug value   */
-#define DM_MM "math_matrix"                 /* Debug message / Error message */
-#define ERROR matrix_error_msg
+extern const char *PRT;
+extern const char *matrix_error_msg[];
+
+#define DV_MM LDBL_MATRIX + 10   /* Debug value   */
+#define DM_MM "math_matrix"      /* Debug message / Error message */
+#define LMM_ERROR matrix_error_msg
 
 /**
  ** debug level
@@ -75,7 +76,7 @@ add(const math_matrix < T > &M, const math_matrix < T > &N)
 			     N.columns, "N.columns", "M.columns == N.columns",
 			     "void math_matrix < T >::"
 			     "add(const math_matrix < T > &M, const math_matrix < T > &N)",
-			     DM_MM, ERROR[4]);
+			     DM_MM, LMM_ERROR[4]);
   
   if (rows != N.rows)
     set_no_of_rows(N.rows);
@@ -183,7 +184,7 @@ subtract(const math_matrix < T > &M, const math_matrix < T > &N)
 			     N.columns, "N.columns", "M.columns == N.columns",
 			     "void math_matrix < T >::"
 			     "subtract(const math_matrix < T > &M, const math_matrix < T > &N)",
-			     DM_MM, ERROR[4]);
+			     DM_MM, LMM_ERROR[4]);
   if (rows != N.rows)
     set_no_of_rows(N.rows);
   if (columns != N.columns)
@@ -286,7 +287,7 @@ multiply(const math_matrix < T > &A, const math_matrix < T > &B)
 			     B.rows, "B.rows", "A.columns == B.rows",
 			     "void math_matrix < T >::"
 			     "multiply(const math_matrix < T > &A, const math_matrix < T > &B)",
-			     DM_MM, ERROR[4]);
+			     DM_MM, LMM_ERROR[4]);
 
   register lidia_size_t j, i, z;
   T TMP, TMP1, *Atmp, *REStmp;
@@ -418,7 +419,7 @@ compwise_multiply(const math_matrix < T > &A, const math_matrix < T > &B)
 			     B.columns, "B.columns", "B.columns == B.columns",
 			     "void math_matrix < T >::"
 			     "compwise_multiply(const math_matrix < T > &A, const math_matrix < T > &B)",
-			     DM_MM, ERROR[4]);
+			     DM_MM, LMM_ERROR[4]);
   
   if (rows != A.rows)
     set_no_of_rows(A.rows);
@@ -459,12 +460,12 @@ multiply_right(math_vector < T > &res, const math_vector < T > &v) const
 			     v.size(), "v.size", "columns == v.size",
 			     "void math_matrix < T >::"
 			     "multiply_right(math_vector < T > &res, const math_vector < T > &v) const",
-			     DM_MM, ERROR[1]);
+			     DM_MM, LMM_ERROR[1]);
 
   register lidia_size_t i, j, l = res.size();
   T TMP, TMP1, *tmp;
   
-  T *tmp1, *tmp2 = v.get_data_address();
+  T *tmp1 = res.get_data_address(), *tmp2 = v.get_data_address();
   
   if (tmp1 == tmp2)
     {
@@ -528,7 +529,7 @@ multiply_right(T *&c, const T *v) const
     lidia_error_handler_para(PRT, "v", "v != NULL",
 			     "void math_matrix < T >::"
 			     "multiply_right(T *c, const T *v) const",
-			     DM_MM, ERROR[6]);
+			     DM_MM, LMM_ERROR[6]);
   
   register lidia_size_t i, j;
   T TMP, TMP1, *tmp;
@@ -573,7 +574,7 @@ multiply_left(math_vector < T > &res, const math_vector < T > &v) const
 			     v.size(), "v.size", "rows == v.size", 
 			     "void math_matrix < T >::"
 			     "multiply_left(math_vector < T > &res, const math_vector < T > &v) const",
-			     DM_MM, ERROR[6]);
+			     DM_MM, LMM_ERROR[6]);
 
   register lidia_size_t i, j, l = res.size();
   T TMP, TMP1;
@@ -639,7 +640,7 @@ multiply_left(T *&c, const T *v) const
     lidia_error_handler_para(PRT, "v", "v != NULL",
 			     "void math_matrix < T >::"
 			     "multiply_left(T *c, const T *v) const",
-			     DM_MM, ERROR[6]);
+			     DM_MM, LMM_ERROR[6]);
 
   register lidia_size_t i, j;
   T TMP, TMP1;
@@ -719,7 +720,7 @@ compwise_divide(const math_matrix < T > &A, const math_matrix < T > &B)
 			     B.columns, "B.columns", "B.columns == B.columns",
 			     "void math_matrix < T >::"
 			     "compwise_divide(const math_matrix < T > &A, const math_matrix < T > &B)",
-			     DM_MM, ERROR[4]);
+			     DM_MM, LMM_ERROR[4]);
   
   if (rows != A.rows)
     set_no_of_rows(A.rows);
@@ -837,7 +838,7 @@ trace(T &tr) const
 			     columns, "columns", "rows == columns",
 			     "void math_matrix < T >::"
 			     "trace(T &tr) const",
-			     DM_MM, ERROR[7]);
+			     DM_MM, LMM_ERROR[7]);
 
   register lidia_size_t i;
   tr = value[0][0];
@@ -845,7 +846,9 @@ trace(T &tr) const
     ::add(tr,tr,value[i][i]);
 }
 
-
+#undef DV_MM 
+#undef DM_MM 
+#undef LMM_ERROR 
 
 
 
