@@ -38,21 +38,22 @@
 #include <LiDIA:sort_vector.h>
 #include <LiDIA:arith.inl>
 #include <LiDIA:xerror.h>
-#include <LiDIA:vector_error_msg.c>
 #else
 #include <LiDIA/sort_vector.h>
 #include <LiDIA/arith.inl>
 #include <LiDIA/xerror.h>
-#include <LiDIA/vector_error_msg.c>
 #endif
 
 /**
  ** debug defines / error defines
  **/
 
-#define DV_SV LDBL_VECTOR + 10          /* Debug value   */
-#define DM_SV "sort_vector"             /* Debug message / Error message */
-#define ERROR vector_error_msg
+extern const char *PRT;
+extern const char *vector_error_msg[];
+
+#define DV_SV LDBL_VECTOR + 10   /* Debug value   */
+#define DM_SV "sort_vector"      /* Debug message / Error message */
+#define LSV_ERROR vector_error_msg
 
 /**
  ** debug level
@@ -106,7 +107,7 @@ set_sort_direction(char sd)
     lidia_error_handler_para(sd, "sd", "sd == SORT_VECTOR_UP or sd == SORT_VECTOR_DOWN",
 			     "void sort_vector < T >::"
 			     "set_sort_direction(char sd)",
-			     DM_SV, ERROR[10]);
+			     DM_SV, LSV_ERROR[10]);
   else
     {
       sort_dir = sd;
@@ -148,7 +149,7 @@ sort(int (*cmp)(const T & a, const T & b), lidia_size_t l, lidia_size_t r)
 			     r, "r", "0 <= r < length",
 			     "void sort_vector < T >::"
 			     "sort(int (*cmp)(const T & a, const T & b), lidia_size_t l, lidia_size_t r)",
-			     DM_SV, ERROR[11]);
+			     DM_SV, LSV_ERROR[11]);
 
   T *ptr = &data[l];
   len = r - l + 1;                      /* length of selected cut */
@@ -185,7 +186,7 @@ sort(char sort_direction, lidia_size_t l, lidia_size_t r)
 			     r, "r", "0 <= r < length",
 			     "void sort_vector < T >::"
 			     "sort(char sort_direction, lidia_size_t l, lidia_size_t r)",
-			     DM_SV, ERROR[11]);
+			     DM_SV, LSV_ERROR[11]);
     
   /* call of corresponding function */
   switch(sort_direction)
@@ -207,7 +208,7 @@ sort(char sort_direction, lidia_size_t l, lidia_size_t r)
 			       "{SORT_VECTOR_UP, SORT_VECTOR_DOWN, SORT_VECTOR_CMP}",
 			       "void sort_vector < T >::"
 			       "sort(char sort_direction, lidia_size_t l, lidia_size_t r)",
-			       DM_SV, ERROR[10]);
+			       DM_SV, LSV_ERROR[10]);
     }
 }
 
@@ -298,7 +299,7 @@ bin_search(const T & x, lidia_size_t & pos, int (*cmp)(const T & a, const T & b)
 			     "bool sort_vector < T >::"
 			     "bin_search(const T & x, lidia_size_t & pos, int (*cmp)(const T & a, const T & b),"
 			     "lidia_size_t l,lidia_size_t r) const",
-			     DM_SV, ERROR[11]);
+			     DM_SV, LSV_ERROR[11]);
 
   /* pick center element */
   pos = (l + r) / 2;
@@ -346,7 +347,7 @@ bin_search(const T & x, lidia_size_t & pos, char sort_direction, lidia_size_t l,
 			      "bool sort_vector < T >::"
 			      "bin_search(const T & x, lidia_size_t & pos, char sort_direction, lidia_size_t l,"
 			      "lidia_size_t r) const",
-			      DM_SV, ERROR[11]);
+			      DM_SV, LSV_ERROR[11]);
   
   switch (sort_direction)
     {
@@ -368,7 +369,7 @@ bin_search(const T & x, lidia_size_t & pos, char sort_direction, lidia_size_t l,
 			       "bool sort_vector < T >::"
 			       "bin_search(const T & x, lidia_size_t &pos, char sort_direction, "
 			       "lidia_size_t l, lidia_size_t r) const",
-			       DM_SV, ERROR[10]);
+			       DM_SV, LSV_ERROR[10]);
     }
   return rc;
 }
@@ -436,29 +437,6 @@ bin_search_down(const T & x, lidia_size_t & pos, lidia_size_t l, lidia_size_t r)
  ** insertion into a sorted vector  
  **/
 
-template < class T >
-void sort_vector < T >::
-insert_at(const T & x, lidia_size_t pos)
-{
-  debug_handler_l(DM_SV, "in member - function "
-		  "insert_at (const T &, lidia_size_t)", DV_SV + 3);
-  
-  register lidia_size_t i, l = length;
-  
-  if (pos < 0)
-    lidia_error_handler_para(pos, "pos", "pos >= 0",
-			     "void sort_vector < T >::"
-			     "insert_at(const T & x, lidia_size_t pos)",
-			     DM_SV , ERROR[1]);
-           
-  set_size((l+1) > (pos + 1) ? l + 1 : pos + 1);
-  
-  /* if size has been increased, insert at position 'pos' */
-  for (i = length - 1; i > pos; i--)
-    data[i] = data[i - 1];
-  data[pos] = x;
-}
-
 template < class T >
 void sort_vector < T >::
 insert(const T & x, int (*cmp)(const T & a, const T & b), lidia_size_t l, lidia_size_t r)
@@ -483,7 +461,7 @@ insert(const T & x, int (*cmp)(const T & a, const T & b), lidia_size_t l, lidia_
 			     "void sort_vector < T >::"
 			     "insert(const T & x, int (*cmp)(const T & a, const T & b), lidia_size_t l, "
 			     "lidia_size_t r)",
-			     DM_SV, ERROR[11]);
+			     DM_SV, LSV_ERROR[11]);
   
   /* examine, where to put element x */
   found = bin_search(x, pos, cmp, l, r);
@@ -519,7 +497,7 @@ insert(const T & x, char sort_direction, lidia_size_t l, lidia_size_t r)
 			     r, "r", "0 <= r < length and r >= l",
 			     "void sort_vector < T >::"
 			     "insert(const T & x, char sort_direction, lidia_size_t l, lidia_size_t r)", 
-			     DM_SV , ERROR[11]);
+			     DM_SV , LSV_ERROR[11]);
   
   /* examine, where to put element x */
   found = bin_search(x, pos, sort_direction, l, r);
@@ -553,7 +531,7 @@ remove(const T & x, int (*cmp)(const T & a, const T & b), lidia_size_t l, lidia_
 			     "bool sort_vector < T >::"
 			     "remove(const T & x, int (*cmp)(const T & a, const T & b), lidia_size_t l, "
 			     "lidia_size_t r)",
-			     DM_SV, ERROR[13]);
+			     DM_SV, LSV_ERROR[13]);
 
   /* determine first occurence of x in the vector */
   found = bin_search(x, pos, cmp, l, r);
@@ -587,7 +565,7 @@ remove(const T &x, char sort_direction, lidia_size_t l, lidia_size_t r)
 			     r, "r", "0 <= r < length and l <= r",
 			     "bool sort_vector < T >::"
 			     "remove(const T &x, char sort_direction, lidia_size_t l, lidia_size_t r)",
-			     DM_SV, ERROR[13]);
+			     DM_SV, LSV_ERROR[13]);
 
   /* determine first occurence of x in the vector */
   found = bin_search(x, pos, sort_direction, l, r);
@@ -662,5 +640,7 @@ delete_copies()
   length = j;
 }
 
-
+#undef DV_SV 
+#undef DM_SV
+#undef LSV_ERROR 
 
