@@ -15,6 +15,10 @@
 #define TIME_MODE 0
 #define HMS_MODE 1
 
+#ifdef sun
+extern "C" {int getrusage(int who, struct rusage *rusage);}
+#endif
+
 class timer 
 {
   private:
