@@ -4,8 +4,37 @@
 //
 // File        : bigint.c (implementation of the gmp interface) 
 // Author      : Thomas Papanikolaou (TP)
-// Last change : TP, Feb 7 1995, initial version
+// Last change : TP, Feb  7 1995, initial version
+//	         MM, Sep 17 1996, 
+//		     In the gmp-1.0 version the interface functions
+//		     rounded to zero. In the gmp-2.0 version these
+//		     functions round to -infinity. Therefore I have
+//		     replaced the interface functions by special
+//		     functions which round to zero:
+//		     -) replaced mpz_div by mpz_tdiv_q 
+//		     -) replaced mpz_divmod by mpz_tdiv_qr
+//		     -) replaced mpz_div_2exp by mpz_tdiv_q_2exp
+//		     
+//		     The gmp-2.0 mpz_mod function always delivers a
+//		     non-negative rest, but LiDIA's rest r has sign:
+//		     sign(r) = sign(a), if r = a % m. Therefore I added an
+//		     if-then to compute the correct sign.
 //
+//		     Modified the code of the power functions to
+//		     handle the special cases (a = +-1) correctly.
+//
+//               MM, Oct 21 1996
+//                   Added test for 0 in randomize.
+//      
+//		 MM, Jan 9 1997
+//		     Exchanged the code for print_to_file and write_to_file
+//		     and also for scan_from_file and read_from_file.
+//
+//		 MM, Jan 10 1997 
+//		     Changed the return value of string_to_bigint to the number of
+//		     read characters instead of simply 0 or -1.
+//
+
 
 #include <LiDIA/bigint.h>
 #include <math.h>
@@ -28,7 +57,7 @@ bigint::bigint(long l)
 bigint::bigint(unsigned long ul)
 { mpz_init_set_ui(&I, ul); }
 
-bigint::bigint(double d)
+void bigint::assign(double d)
 {
   MP_INT tmp;
   double mantisse;
@@ -65,7 +94,7 @@ bigint::bigint(double d)
     if (exponent >= 0)
         mpz_mul_2exp(&I, &I, exponent);
     else
-        mpz_div_2exp(&I, &I, -exponent);
+        mpz_tdiv_q_2exp(&I, &I, -exponent);
     if (sign)
       mpz_neg(&I, &I);
   }
@@ -83,11 +112,16 @@ bigint::~bigint()
 **/
 
 int bigint::bit(unsigned int i) const
-{ }
+{ 
+  if (i >= bit_length()) 
+    return 0;
+  else
+    return ((I._mp_d[i/BITS_PER_MP_LIMB] >> (i % BITS_PER_MP_LIMB)) & 1);
+}
 
 int bigint::length() const
 { 
-  int i = (int)I.size;
+  int i = (int)I._mp_size;
   if (i < 0) i = -i;
   return i;
 } 
@@ -98,7 +132,7 @@ int bigint::bit_length() const
   if (i)
   {
     int cnt;
-    count_leading_zeros (cnt, I.d[i - 1]);
+    count_leading_zeros (cnt, I._mp_d[i - 1]);
     i = i * BITS_PER_MP_LIMB - cnt;
   }
   return i;
@@ -106,43 +140,43 @@ int bigint::bit_length() const
 
 int bigint::sign() const
 {
-  if (I.size < 0)
+  if (I._mp_size < 0)
     return -1;
   else
-    return (I.size != 0); 
+    return (I._mp_size != 0); 
 } 
 
-int bigint::is_positive() const
-{ return (I.size > 0); } 
+bool bigint::is_positive() const
+{ return (I._mp_size > 0); } 
 
-int bigint::is_negative() const
-{ return (I.size < 0); }
+bool bigint::is_negative() const
+{ return (I._mp_size < 0); }
 
-int bigint::is_even() const
-{ return (I.size == 0 || (!(I.d[0] & 1))); }
+bool bigint::is_even() const
+{ return (I._mp_size == 0 || (!(I._mp_d[0] & 1))); }
 
-int bigint::is_odd() const
-{ return (I.size && (I.d[0] & 1)); }
+bool bigint::is_odd() const
+{ return (I._mp_size && (I._mp_d[0] & 1)); }
 
-int bigint::is_zero() const
-{ return (I.size == 0); }
+bool bigint::is_zero() const
+{ return (I._mp_size == 0); }
 
-int bigint::is_gt_zero() const
-{ return (I.size > 0); }
+bool bigint::is_gt_zero() const
+{ return (I._mp_size > 0); }
 
-int bigint::is_ge_zero() const
-{ return (I.size >= 0); }
+bool bigint::is_ge_zero() const
+{ return (I._mp_size >= 0); }
 
-int bigint::is_lt_zero() const
-{ return (I.size < 0); }
+bool bigint::is_lt_zero() const
+{ return (I._mp_size < 0); }
 
-int bigint::is_le_zero() const
-{ return (I.size <= 0); }
+bool bigint::is_le_zero() const
+{ return (I._mp_size <= 0); }
 
-int bigint::is_one() const
-{ return ((I.size == 1) && (I.d[0] == 1)); }
+bool bigint::is_one() const
+{ return ((I._mp_size == 1) && (I._mp_d[0] == 1)); }
 
-int bigint::intify(int & i) const
+bool bigint::intify(int & i) const
 { 
   if (bit_length() > (BITS_PER_INT-1))
     return 1;
@@ -150,7 +184,7 @@ int bigint::intify(int & i) const
     return 0;
 }
 
-int bigint::longify(long & i) const
+bool bigint::longify(long & i) const
 { 
   if (bit_length() > (BITS_PER_LONGINT-1))
     return 1;
@@ -160,12 +194,12 @@ int bigint::longify(long & i) const
 
 int bigint::abs_compare(const bigint & a) const
 {
-  mp_size_t this_size = ABS(I.size);
-  mp_size_t a_size = ABS(a.I.size);
+  mp_size_t this_size = ABS(I._mp_size);
+  mp_size_t a_size = ABS(a.I._mp_size);
   if (this_size > a_size) return 1;
   if (this_size < a_size) return -1;
   if (this_size == 0) return 0;
-  mp_srcptr tp = I.d, ap = a.I.d;
+  mp_srcptr tp = I._mp_d, ap = a.I._mp_d;
   return __mpn_cmp (tp, ap, this_size);
 }
 
@@ -179,18 +213,18 @@ int bigint::compare(const bigint & a) const
 unsigned long bigint::most_significant_digit() const
 { 
   unsigned long msd;
-  long l = I.size;
+  long l = I._mp_size;
   if (l == 0) return 0;
   l = (l < 0) ? -l : l;
-  msd = I.d[l - 1];
+  msd = I._mp_d[l - 1];
   return msd;
 }
 
 unsigned long bigint::least_significant_digit() const
 {
   unsigned long lsd;
-  if (I.size == 0) return 0;
-  lsd = I.d[0];
+  if (I._mp_size == 0) return 0;
+  lsd = I._mp_d[0];
   return lsd;
 }
 
@@ -228,34 +262,34 @@ void bigint::multiply_by_2()
 { mpz_mul_2exp(&I, &I, 1); }
 
 void bigint::divide_by_2()
-{ mpz_div_2exp(&I, &I, 1); }
+{ mpz_tdiv_q_2exp(&I, &I, 1); }
 
 /**
 ** Type checking
 **/
 
-int is_char(const bigint & a)
+bool is_char(const bigint & a)
 { return a.bit_length() <= (BITS_PER_CHAR - 1); }
 
-int is_uchar(const bigint & a)
+bool is_uchar(const bigint & a)
 { return a.bit_length() <= (BITS_PER_CHAR); }
 
-int is_short(const bigint & a)
+bool is_short(const bigint & a)
 { return a.bit_length() <= (BITS_PER_SHORTINT - 1); }
 
-int is_ushort(const bigint & a)
+bool is_ushort(const bigint & a)
 { return a.bit_length() <= (BITS_PER_SHORTINT); }
 
-int is_int(const bigint & a)
+bool is_int(const bigint & a)
 { return a.bit_length() <= (BITS_PER_INT - 1); }
 
-int is_uint(const bigint & a)
+bool is_uint(const bigint & a)
 { return a.bit_length() <= (BITS_PER_INT); }
 
-int is_long(const bigint & a)
+bool is_long(const bigint & a)
 { return a.bit_length() <= (BITS_PER_LONGINT - 1); }
 
-int is_ulong(const bigint & a)
+bool is_ulong(const bigint & a)
 { return a.bit_length() <= (BITS_PER_LONGINT); }
 
 /**
@@ -311,11 +345,11 @@ double bigint::operator = (double d)
     if (exponent >= 0)
         mpz_mul_2exp(&I, &I, exponent);
     else
-        mpz_div_2exp(&I, &I, -exponent);
+        mpz_tdiv_q_2exp(&I, &I, -exponent);
     if (sign)
       mpz_neg(&I, &I);
+    mpz_clear(&tmp);
   }
-  mpz_clear(&tmp);
   return d;
 } 
 
@@ -326,22 +360,22 @@ bigint & bigint::operator = (const bigint & a)
 ** comparisons
 **/
 
-int operator == (const bigint & a, const bigint & b)
+bool operator == (const bigint & a, const bigint & b)
 { return mpz_cmp(&a.I, &b.I) == 0;}
  
-int operator != (const bigint & a, const bigint & b)
+bool operator != (const bigint & a, const bigint & b)
 { return mpz_cmp(&a.I, &b.I) != 0;}
  
-int operator > (const bigint & a, const bigint & b)
+bool operator > (const bigint & a, const bigint & b)
 { return mpz_cmp(&a.I, &b.I) > 0;}
  
-int operator >= (const bigint & a, const bigint & b)
+bool operator >= (const bigint & a, const bigint & b)
 { return mpz_cmp(&a.I, &b.I) >= 0;}
  
-int operator < (const bigint & a, const bigint & b)
+bool operator < (const bigint & a, const bigint & b)
 { return mpz_cmp(&a.I, &b.I) < 0;}
  
-int operator <= (const bigint & a, const bigint & b)
+bool operator <= (const bigint & a, const bigint & b)
 { return mpz_cmp(&a.I, &b.I) <= 0;}
 
 /**
@@ -361,10 +395,23 @@ bigint operator * (const bigint & a, const bigint & b)
 { bigint c; mpz_mul(&c.I, &a.I, &b.I); return c; }
 
 bigint operator / (const bigint & a, const bigint & b)
-{ bigint c; mpz_div(&c.I, &a.I, &b.I); return c; }
+{ bigint c; mpz_tdiv_q(&c.I, &a.I, &b.I); return c; }
 
 bigint operator % (const bigint & a, const bigint & b)
-{ bigint c; mpz_mod(&c.I, &a.I, &b.I); return c; }
+{ 
+  bigint c;
+  mpz_mod(&c.I, &a.I, &b.I); // returns non-negative rest c
+
+  if (a.I._mp_size < 0 && c.I._mp_size != 0)
+   {
+     // a is negative and rest != 0 -> create negative rest
+     if (b.I._mp_size < 0)
+        mpz_add(&c.I, &c.I, &b.I);
+     else
+        mpz_sub(&c.I, &c.I, &b.I);
+   }
+  return c;
+}
 
 bigint operator << (const bigint & a, long ui)
 { 
@@ -380,7 +427,7 @@ bigint operator >> (const bigint & a, long ui)
   bigint c; 
   if (ui < 0) 
     lidia_error_handler("bigint", "operator>>::index is negative.");
-  mpz_div_2exp(&c.I, &a.I, ui); 
+  mpz_tdiv_q_2exp(&c.I, &a.I, ui); 
   return c; 
 }
 
@@ -403,10 +450,53 @@ bigint & bigint::operator *= (const bigint & a)
 { mpz_mul(&I, &I, &a.I); return *this; }
 
 bigint & bigint::operator /= (const bigint & a)
-{ mpz_div(&I, &I, &a.I); return *this; }
+{ mpz_tdiv_q(&I, &I, &a.I); return *this; }
+
 
 bigint & bigint::operator %= (const bigint & a)
-{ mpz_mod(&I, &I, &a.I); return *this; }
+{
+  // can be optimized by using static tmp
+  //
+  // Problem: see first MM-entry in Last Changes, mpz_mod
+
+  if (I._mp_size < 0) // rest must be negative
+   {
+     if (&a == this)  // a will be overwritten
+      {
+	MP_INT tmp;   // set tmp = |a|
+	if (a.I._mp_size < 0) // a is negative
+	  {
+	    mpz_init(&tmp);
+	    mpz_neg(&tmp,&a.I);
+	  }
+	else
+	    mpz_init_set(&tmp,&a.I);
+
+        mpz_mod(&I, &I, &a.I); // compute non-negative rest
+
+	if (I._mp_size != 0) // rest is not equal to zero
+	 {
+	   mpz_sub(&I, &I, &tmp); // create negative rest
+  	   mpz_clear(&tmp);
+	 }
+      }
+     else // a won't be overwritten
+      {
+        mpz_mod(&I, &I, &a.I); // compute non-negative rest
+
+        if (I._mp_size != 0) // rest is not equal to zero
+         {
+	   if (a.I._mp_size < 0)  // create negative rest
+              mpz_add(&I, &I, &a.I);
+	   else
+	      mpz_sub(&I, &I, &a.I);
+         }
+      }
+   }
+  else
+     mpz_mod(&I, &I, &a.I);
+  return *this;
+}
 
 bigint & bigint::operator <<= (long ui)
 { 
@@ -420,7 +510,7 @@ bigint & bigint::operator >>= (long ui)
 { 
   if (ui < 0) 
     lidia_error_handler("bigint", "operator>>=::index is negative.");
-  mpz_div_2exp(&I, &I, ui);
+  mpz_tdiv_q_2exp(&I, &I, ui);
   return *this;
 }
 
@@ -439,14 +529,14 @@ bigint & bigint::operator++ ()
 bigint & bigint::operator-- ()
 { mpz_sub_ui(&I, &I, 1); return *this; }
 
-bigint & bigint::operator++ (int)
-{ mpz_add_ui(&I, &I, 1); return *this; }
+bigint  bigint::operator++ (int)
+{ bigint a = *this; mpz_add_ui(&I, &I, 1); return a; }
 
-bigint & bigint::operator-- (int)
-{ mpz_sub_ui(&I, &I, 1); return *this; }
+bigint  bigint::operator-- (int)
+{ bigint a =*this; mpz_sub_ui(&I, &I, 1); return a; }
 
 int bigint::operator ! () const
-{ return (I.size == 0); }
+{ return (I._mp_size == 0); }
 
 bigint bigint::operator ~ () const
 { bigint c; mpz_com(&c.I, &I); return c; }
@@ -468,17 +558,59 @@ void multiply(bigint & c, const bigint & a, const bigint & b)
 { mpz_mul(&c.I, &a.I, &b.I); }
 
 void divide(bigint & c, const bigint & a, const bigint & b)
-{ mpz_div(&c.I, &a.I, &b.I); }
+{ mpz_tdiv_q(&c.I, &a.I, &b.I); }
 
 void remainder(bigint & c, const bigint & a, const bigint & b)
-{ mpz_mod(&c.I, &a.I, &b.I); }
+{ 
+  // can be optimized by using static tmp
+  //
+  // Problem: see first MM-entry in Last Changes, mpz_mod
+
+  if (a.I._mp_size < 0) // rest must be negative
+   {
+     if (&b == &c)  // b will be overwritten
+      {
+	MP_INT tmp;   // set tmp = |b|
+	if (b.I._mp_size < 0) // b is negative
+	  {
+	    mpz_init(&tmp);
+	    mpz_neg(&tmp,&b.I);
+	  }
+	else
+	    mpz_init_set(&tmp,&b.I);
+
+        mpz_mod(&c.I, &a.I, &b.I); // compute non-negative rest
+
+	if (c.I._mp_size != 0) // rest is not equal to zero
+	 {
+	   mpz_sub(&c.I, &c.I, &tmp); // create negative rest
+  	   mpz_clear(&tmp);
+	 }
+      }
+     else // b won't be overwritten
+      {
+        mpz_mod(&c.I, &a.I, &b.I); // compute non-negative rest
+
+        if (c.I._mp_size != 0) // rest is not equal to zero
+         {
+	   if (b.I._mp_size < 0)  // create negative rest
+              mpz_add(&c.I, &c.I, &b.I);
+	   else
+	      mpz_sub(&c.I, &c.I, &b.I);
+         }
+      }
+   }
+  else
+     mpz_mod(&c.I, &a.I, &b.I);
+}
+
 
 void div_rem(bigint & q, bigint & r, const bigint & a, const bigint & b)
-{ mpz_divmod(&q.I, &r.I, &a.I, &b.I); }
+{ mpz_tdiv_qr(&q.I, &r.I, &a.I, &b.I); }
 
 void invert(bigint & a, const bigint & b)
 {
-  if ((a.I.size == 1 || a.I.size == -1) && a.I.d[0] == 1)
+  if ((a.I._mp_size == 1 || a.I._mp_size == -1) && a.I._mp_d[0] == 1)
     mpz_set(&a.I, &b.I);
   else
     lidia_error_handler("bigint", "invert::inverting of a non-unit.");
@@ -495,15 +627,24 @@ void shift_right(bigint & c, const bigint & a, long ui)
 { 
   if (ui < 0) 
     lidia_error_handler("bigint", "shift_right()::index is negative.");
-  mpz_div_2exp(&c.I, &a.I, ui);
+  mpz_tdiv_q_2exp(&c.I, &a.I, ui);
 }
 
 void power(bigint & c, const bigint & a, const bigint & b)
 {
   bigint exponent, multiplier;
-  if (b.is_negative())
+  if (a.I._mp_d[0]  ==  1 &&
+      (a.I._mp_size ==  1 ||
+       a.I._mp_size == -1 )) // a = +-1 -> a^(|b|)
+    {
+      if (b.is_odd())
+	c = a;
+      else
+	c.assign_one();
+    }
+  else if (b.is_negative()) // b < 0, a != +-1 -> 0
     c.assign_zero();
-  else if (b.is_zero() || a.is_one())
+  else if (b.is_zero()) // b == 0 -> 1
     c.assign_one();
   else
   {
@@ -522,11 +663,20 @@ void power(bigint & c, const bigint & a, const bigint & b)
 
 void power(bigint & c, const bigint & a, long i)
 {
-  if (i == 0 || a.is_one())
-    c.assign_one(); 
-  else if (i < 0)
-    c.assign_one(); 
-  else
+  if (a.I._mp_d[0]  ==  1 &&
+      (a.I._mp_size ==  1 ||
+       a.I._mp_size == -1 )) // a = +-1 -> a^(|i|)
+    {
+      if (i&1)
+	c = a;
+      else
+	c.assign_one();
+    }
+  else if (i < 0)  // i < 0, a != +-1 -> 0
+    c.assign_zero();
+  else if (i == 0) // i == 0 -> 1
+    c.assign_one();
+  else 
     mpz_pow_ui(&c.I, &a.I, i); 
 } 
 
@@ -579,7 +729,7 @@ void divide(bigint & c, const bigint & a, long i)
 { // optimize this by making tmp static
   MP_INT tmp;
   mpz_init_set_si(&tmp, i);
-  mpz_div(&c.I, &a.I, &tmp);
+  mpz_tdiv_q(&c.I, &a.I, &tmp);
   mpz_clear(&tmp);
 }
   
@@ -589,10 +739,18 @@ void remainder(long &r, const bigint & a, long i)
   MP_INT tmp1, tmp2;
   mpz_init_set_si(&tmp1, i);
   mpz_init(&tmp2);
-  mpz_mod(&tmp2, &a.I, &tmp1);
+  mpz_mod(&tmp2, &a.I, &tmp1); // compute non-negative rest r
   r = mpz_get_si(&tmp2);
   mpz_clear(&tmp2);
   mpz_clear(&tmp1);
+
+  if (r != 0 && a.I._mp_size < 0) // r must be negative
+   {
+     if (i < 0) 
+      r += i;
+     else
+      r -= i;
+   }
 }
 
 long remainder(const bigint & a, long i)
@@ -601,10 +759,19 @@ long remainder(const bigint & a, long i)
   MP_INT tmp1, tmp2;
   mpz_init_set_si(&tmp1, i);
   mpz_init(&tmp2);
-  mpz_mod(&tmp2, &a.I, &tmp1);
+  mpz_mod(&tmp2, &a.I, &tmp1); // compute non-negative rest r
   r = mpz_get_si(&tmp2);
   mpz_clear(&tmp2);
   mpz_clear(&tmp1);
+
+  if (r != 0 && a.I._mp_size < 0) // r must be negative
+   {
+     if (i < 0) 
+      r += i;
+     else
+      r -= i;
+   }
+
   return r;
 }
 
@@ -613,7 +780,7 @@ void div_rem(bigint & q, long &r, const bigint & a, long i)
   MP_INT tmp1, tmp2;
   mpz_init_set_si(&tmp1, i);
   mpz_init(&tmp2);
-  mpz_divmod(&q.I, &tmp2, &a.I, &tmp1);
+  mpz_tdiv_qr(&q.I, &tmp2, &a.I, &tmp1);
   r = mpz_get_si(&tmp2);
   mpz_clear(&tmp2);
   mpz_clear(&tmp1);
@@ -646,13 +813,33 @@ bigint xgcd_right(bigint & v, const bigint & a, const bigint & b)
 **/
 
 bigint abs(const bigint & a)
-{ bigint c(a); c.I.size = ABS(c.I.size); return c; }
+{ bigint c(a); c.I._mp_size = ABS(c.I._mp_size); return c; }
 
 void seed(const bigint & a)
 { bigint c; mpz_random(&c.I, 1); }
 
 bigint randomize(const bigint & a)
-{ bigint c; mpz_random(&c.I, mpz_size(&a.I)); return c; }
+{ 
+  bigint c; 
+
+  if (a.is_zero())
+     lidia_error_handler("bigint", "Bound must not be equal to zero.");
+  else
+   {
+     mpz_random(&c.I, mpz_size(&a.I)); 
+     if (c.is_lt_zero())
+      {
+        if (c < a)
+	    remainder(c,c,a);
+      }
+     else if (c > a)
+	remainder(c,c,a);
+   }
+
+  return c; 
+}
+
+#ifdef USE_OLD_DBL
 
 double dbl(const bigint & a)
 {
@@ -666,6 +853,70 @@ double dbl(const bigint & a)
   return d;
 }
 
+xdouble xdbl(const bigint & a)
+{
+  char *s;
+  int t = (int) mpz_sizeinbase(&a.I, 10) + 2;
+  s = new char [t];
+  mpz_get_str(s, 10, &a.I);
+  xdouble d;
+  d = string_to_xdouble(s);
+  delete[] s;
+  return d;
+}
+
+#else
+
+double dbl(const bigint & a)
+{
+  long l = a.I._mp_size;
+  l=((l<0)?-l:l);
+  double d = 0.0;
+  if (l)
+  {
+    int i = 1;
+    d = a.I._mp_d[0]; 
+    double base = bigint::radix();
+    double power = base;
+    while (i < l)
+    {
+     d += a.I._mp_d[i] * power;
+     power *= base;
+     i++;
+    }
+    if (a.I._mp_size < 0)
+      d = -d;
+  }
+  return d;
+}
+
+xdouble xdbl(const bigint & a)
+{
+  long l = a.I._mp_size;
+  l=((l<0)?-l:l);
+  xdouble d = 0.0;
+  if (l)
+  {
+    int i = 1;
+    d = (double )a.I._mp_d[0]; 
+    xdouble base = bigint::radix();
+    xdouble power = base;
+    while (i < l)
+    {
+     d += a.I._mp_d[i] * power;
+     power *= base;
+     i++;
+    }
+    if (a.I._mp_size < 0)
+      d = -d;
+  }
+  return d;
+}
+
+#endif
+
+
+
 void sqrt(bigint & a, const bigint & b)
 { mpz_sqrt(&a.I, &b.I); }
       
@@ -746,8 +997,13 @@ ostream & operator << (ostream & out, const bigint & a)
   return out;
 }
 
-int string_to_bigint(char *s, bigint & a)
-{ return mpz_set_str(&a.I, s, 0); }
+int string_to_bigint(const char *s, bigint & a)
+{ 
+  if (!mpz_set_str(&a.I,(char*)s, 10))
+    return (strlen(s));
+  else
+    return 0;
+}
 
 int bigint_to_string(const bigint & a, char *s)
 { mpz_get_str(s, 10, &a.I); return strlen(s); }
@@ -757,18 +1013,17 @@ int bigint_to_string(const bigint & a, char *s)
 **/
 
 void bigint::read_from_file(FILE * fp)
-{ mpz_inp_str(&I, fp, 10); }
+{ mpz_inp_raw(&I, fp); }
 
 void bigint::write_to_file(FILE * fp)
-{ mpz_out_str(fp, 10, &I); }
+{ mpz_out_raw(fp, &I); }
 
 /**
 ** using fscanf/fprintf
 **/
 
 void bigint::scan_from_file(FILE * fp)
-{ mpz_inp_raw(&I, fp); }
+{ mpz_inp_str(&I, fp, 10); }
 
 void bigint::print_to_file(FILE * fp)
-{ mpz_out_raw(fp, &I); }
-
+{ mpz_out_str(fp, 10, &I); }
