@@ -28,23 +28,31 @@
 **
 **/
 
-/*
+/* We need the define, because gmp-impl.h does not
+   set a define and udigit_def.h also must include
+   those header files. */
+
+#ifndef LIDIA_HAVE_GMP_H
+#define LIDIA_HAVE_GMP_H
 extern "C" {
 #include <LiDIA/gmp.h>
 #include <LiDIA/gmp-impl.h>
 #include <LiDIA/longlong.h>
 #include <LiDIA/gmp-mparam.h>
 };
-*/
+#endif
 
+/*
 extern "C" {
 #include "gmp.h"
 #include "gmp-impl.h"
 #include "longlong.h"
 #include "gmp-mparam.h"
 };
+*/
+
 
 #define integer_type_name       MP_INT
-#define base_digit       mp_limb
+#define base_digit       mp_limb_t
 
 #endif
