@@ -5,7 +5,23 @@
 // File        : bigint.c (implementation of the libI interface) 
 // Author      : Thomas Papanikolaou (TP)
 // Last change : TP, Feb 7 1995, initial version
-//
+//               MM, Sep 17 1996, 
+//                   Modified the code of the power functions to
+//                   handle the special cases (a = +-1) correctly.
+//		 AH, Sep 25 1996,
+//		     Changed the return-value type of all 
+//		     yes/no-functions to bool. 
+//		 MM, Sep 25 1996
+//                   Changed the return-value type of the
+//                   postfix operators from bigint & to bigint
+//                   because the return-value must be a local variable.
+//		 MM, Sep 26 1996
+//		     Check for zero in divide_by_2() to handle -1 correctly.
+//		 MM, Oct 21 1996
+//	    	     Added test for 0 in randomize.
+//	         VM, Oct 31 1996
+//		     Changed code in multiply and square for greater efficiency
+
 
 #include <LiDIA/bigint.h>
 
@@ -25,7 +41,7 @@ bigint::bigint(long l)
 bigint::bigint(unsigned long ul)
 { cIasulong(&I, ul); }
 
-bigint::bigint(double d)
+void bigint::assign(double d)
 {
   cImaxlength(&I, sizeof(double)/sizeof(DigitType));
   Iasdbl(&I, d); 
@@ -48,65 +64,75 @@ return ((I.vec[i/BitsPerDigit] >> (i % BitsPerDigit)) & 1);
 }
 
 
-int bigint::length() const
+lidia_size_t bigint::length() const
 { return I.length; }
 
-int bigint::bit_length() const
+lidia_size_t bigint::bit_length() const
 { return Ilog(&I) + 1; }
 
 int bigint::sign() const
 { return ((I.sign) ? -1 : (I.length != 0)); }
 
-int bigint::is_positive() const
-{ return (sign() > 0); }
+bool bigint::is_positive() const
+{ if (sign() > 0) return true;
+  return false; }
 
-int bigint::is_negative() const
-{ return (sign() < 0); }
+bool bigint::is_negative() const
+{ if (sign() < 0) return true;
+  return false; }
 
-int bigint::is_even() const
-{ return ((I.length == 0) || (!(*(I.vec) & 1))); }
+bool bigint::is_even() const
+{ if ((I.length == 0) || (!(*(I.vec) & 1))) return true;
+  return false; }
 
-int bigint::is_odd() const
-{ return (I.length && (*(I.vec) & 1)); }
+bool bigint::is_odd() const
+{ if (I.length && (*(I.vec) & 1)) return true;
+  return false; }
 
-int bigint::is_zero() const
-{ return (!I.length); }
+bool bigint::is_zero() const
+{ if (!I.length) return true;
+  return false; }
 
-int bigint::is_gt_zero() const
-{ return ((I.sign == PLUS) && I.length); }
+bool bigint::is_gt_zero() const
+{ if ((I.sign == PLUS) && I.length) return true;
+  return false; }
 
-int bigint::is_ge_zero() const
-{ return (I.sign == PLUS); }
+bool bigint::is_ge_zero() const
+{ if (I.sign == PLUS) return true;
+  return false; }
 
-int bigint::is_lt_zero() const
-{ return (I.sign == MINUS); }
+bool bigint::is_lt_zero() const
+{ if (I.sign == MINUS) return true;
+  return false; }
 
-int bigint::is_le_zero() const
-{ return ((!I.length) || (I.sign == MINUS)); }
+bool bigint::is_le_zero() const
+{ if ((!I.length) || (I.sign == MINUS)) return true;
+  return false; }
 
-int bigint::is_one() const
-{ return ((I.sign == PLUS) && (I.length == 1) && (*(I.vec) == 1)); }
+bool bigint::is_one() const
+{ if ((I.sign == PLUS) && (I.length == 1) && (*(I.vec) == 1)) return true;
+  return false; }
 
-int bigint::intify(int & i) const
+bool bigint::intify(int & i) const
 { 
   if (Iisint(&I)) 
   { 
     i = intasI(&I); 
-    return 0; 
+    return false; 
   }
   else
-    return 1;
+    return true;
 }
 
-int bigint::longify(long & i) const
+bool bigint::longify(long & i) const
 { 
   if (Iislong(&I)) 
   { 
     i = longasI(&I); 
-    return 0; 
+    return false; 
   }
   else
-    return 1;
+    return true;
 }
 
 int bigint::abs_compare(const bigint & a) const
@@ -163,6 +189,9 @@ const int bigint::bits_per_digit()
 void bigint::absolute_value()
 { I.sign = PLUS; }
 
+void bigint::abs()
+{ I.sign = PLUS; }
+
 void bigint::negate()
 { if (I.length) I.sign ^= MINUS; }
 
@@ -188,35 +217,43 @@ void bigint::multiply_by_2()
 { IslasD(&I, 1); }
 
 void bigint::divide_by_2()
-{ Isr1(&I); }
+{ Isr1(&I); if (I.length == 0) I.sign=PLUS; }
 
 /**
 ** Type checking
 **/
 
-int is_char(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= 7); }
+bool is_char(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= 7) return true;
+  return false; }
 
-int is_uchar(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= 8); }
+bool is_uchar(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= 8) return true;
+  return false; }
 
-int is_short(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= (8 * sizeof(short)-1));}
+bool is_short(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= (8 * sizeof(short)-1)) return true;
+  return false; }
 
-int is_ushort(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= (8 * sizeof(unsigned short))); }
+bool is_ushort(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= (8 * sizeof(unsigned short))) return true;
+  return false; }
 
-int is_int(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= (8 * sizeof(int)-1));}
+bool is_int(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= (8 * sizeof(int)-1)) return true;
+  return false; }
 
-int is_uint(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= (8 * sizeof(unsigned int))); }
+bool is_uint(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= (8 * sizeof(unsigned int))) return true;
+  return false; }
 
-int is_long(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= (8 * sizeof(long)-1));}
+bool is_long(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= (8 * sizeof(long)-1)) return true;
+  return false; }
 
-int is_ulong(const bigint & a)
-{ return ((Ilog(&a.I) + 1) <= (8 * sizeof(unsigned long))); }
+bool is_ulong(const bigint & a)
+{ if ((Ilog(&a.I) + 1) <= (8 * sizeof(unsigned long))) return true;
+  return false; }
 
 /**
 ** assignments
@@ -242,23 +279,23 @@ bigint & bigint::operator = (const bigint & a)
 ** comparisons
 **/
 
-int operator == (const bigint & a, const bigint & b)
-{ return IeqI(&a.I, &b.I); }
+bool operator == (const bigint & a, const bigint & b)
+{ if (IeqI(&a.I, &b.I)) return true; else return false; }
 
-int operator != (const bigint & a, const bigint & b)
-{ return !IeqI(&a.I, &b.I); }
+bool operator != (const bigint & a, const bigint & b)
+{ if (IeqI(&a.I, &b.I)) return false; else return true; }
 
-int operator > (const bigint & a, const bigint & b)
-{ return IgtI(&a.I, &b.I); }
+bool operator > (const bigint & a, const bigint & b)
+{ if (IgtI(&a.I, &b.I)) return true; else return false; }
 
-int operator >= (const bigint & a, const bigint & b)
-{ return (IgtI(&a.I, &b.I) || IeqI(&a.I, &b.I)); }
+bool operator >= (const bigint & a, const bigint & b)
+{ if (IgtI(&a.I, &b.I) || IeqI(&a.I, &b.I)) return true; else return false; }
 
-int operator < (const bigint & a, const bigint & b)
-{ return IgtI(&b.I, &a.I); }
+bool operator < (const bigint & a, const bigint & b)
+{ if (IgtI(&b.I, &a.I)) return true; else return false; }
 
-int operator <= (const bigint & a, const bigint & b)
-{ return (IgtI(&b.I, &a.I) || IeqI(&a.I, &b.I)); }
+bool operator <= (const bigint & a, const bigint & b)
+{ if (IgtI(&b.I, &a.I) || IeqI(&a.I, &b.I)) return true; else return false; }
 
 /**
 ** operator overloading
@@ -367,11 +404,11 @@ bigint & bigint::operator++ ()
 bigint & bigint::operator-- ()
 { Idec(&I); return *this; }
 
-bigint & bigint::operator++ (int)
-{ Iinc(&I); return *this; }
+bigint bigint::operator++ (int)
+{ bigint a = *this; Iinc(&I); return a; }
 
-bigint & bigint::operator-- (int)
-{ Idec(&I); return *this; }
+bigint bigint::operator-- (int)
+{ bigint a = *this; Idec(&I); return a; }
 
 int bigint::operator ! () const
 { return (I.length == 0); }
@@ -394,12 +431,6 @@ void subtract(bigint & c, const bigint & a, const bigint & b)
 
 void multiply(bigint & c, const bigint & a, const bigint & b)
 {
-  if (&a == &b)
-  {
-    IasI(&c.I, &a.I);
-    ImuasI(&c.I, &a.I);
-  }
-  else
     IasImuI(&c.I, &a.I, &b.I);
 }
 
@@ -458,31 +489,47 @@ void shift_right(bigint & c, const bigint & a, long ui)
 void power(bigint & c, const bigint & a, const bigint & b)
 {
   bigint exponent, multiplier;
-  if (b.is_negative())
+  if ((a.I.length == 1) &&
+      (*(a.I.vec) == 1)  ) // a = +-1 -> a^(|b|)
+    {
+      if (b.is_odd())
+        c = a;
+      else
+        c.assign_one();
+    }
+  else if (b.is_negative()) // b < 0, a != +-1 -> 0
     c.assign_zero();
-  else if (b.is_zero() || a.is_one())
+  else if (b.is_zero()) // b == 0 -> 1
     c.assign_one();
   else
-  {
+    {
    exponent.assign(b);
    multiplier.assign(a);
    c.assign_one();
    while (exponent.is_gt_zero())
-   {
+     {
     if (!exponent.is_even())
       multiply(c, c, multiplier);
     square(multiplier, multiplier);
     exponent.divide_by_2();
-   }
-  }
+     }
+    }
 }
 
 void power(bigint & c, const bigint & a, long i)
 {
-  if (i == 0 || a.is_one())
-    Ias1(&c.I);
-  else if (i < 0)
-    Ias0(&c.I);
+  if ((a.I.length == 1) &&
+      (*(a.I.vec) == 1)  ) // a = +-1 -> a^(|i|)
+    {
+      if (i&1)
+        c = a;
+      else
+        c.assign_one();
+    }
+  else if (i < 0) // i < 0, a != +-1 -> 0
+    c.assign_zero();
+  else if (i == 0) // i == 0 -> 1
+    c.assign_one();
   else 
     IasIpowD(&c.I, &a.I, i); 
 } 
@@ -611,7 +658,16 @@ void seed(const bigint & a)
 { bigint c; IasrandomI(&c.I, &a.I); }
 
 bigint randomize(const bigint & a)
-{ bigint c; IasrandomI(&c.I, &a.I); return c; }
+{ 
+  bigint c; 
+ 
+  if (a.is_zero())
+     lidia_error_handler("bigint", "Bound must not be equal to zero.");
+  else
+     IasrandomI(&c.I, &a.I);
+ 
+  return c; 
+}
 
 #ifdef USE_OLD_DBL
 
@@ -625,6 +681,16 @@ double dbl(const bigint & a)
   return d;
 }
 
+xdouble xdbl(const bigint & a)
+{
+  xdouble d; 
+  char *s = new char[Ilog(&a.I)/3+10];
+  Itoa(&a.I, s); 
+  d = string_to_xdouble(s);
+  delete[] s;
+  return d;
+}
+
 #else
 
 double dbl(const bigint & a)
@@ -649,13 +715,35 @@ double dbl(const bigint & a)
   return d;
 }
 
+xdouble xdbl(const bigint & a)
+{
+  int l = a.I.length;
+  xdouble d = 0.0;
+  if (l)
+  {
+    int i = 1;
+    d = (double )a.I.vec[0]; 
+    xdouble base = bigint::radix();
+    xdouble power = base;
+    while (i < l)
+    {
+     d += a.I.vec[i] * power;
+     power *= base;
+     i++;
+    }
+    if (a.I.sign == 1)
+      d = -d;
+  }
+  return d;
+}
+
 #endif
 
 void sqrt(bigint & a, const bigint & b)
 { IassqrtI(&a.I, &b.I); }
       
 void square(bigint & a, const bigint & b)
-{ IasI(&a.I, &b.I); ImuasI(&a.I, &b.I);  }
+{ IasImuI(&a.I, &b.I, &b.I); }
 
 void swap(bigint & a, bigint & b)
 {
@@ -744,8 +832,8 @@ ostream & operator << (ostream & out, const bigint & a)
   return out;
 }
 
-int string_to_bigint(char *s, bigint & a)
-{ return atoI(s, &a.I); }
+int string_to_bigint(const char *s, bigint & a)
+{ return atoI((char*)s, &a.I); }
 
 int bigint_to_string(const bigint & a, char *s)
 { return Itoa(&a.I, s); }
@@ -761,10 +849,10 @@ void bigint::read_from_file(FILE * fp)
   soD = sizeof(DigitType);
   if (feof(fp)) return;
   delDigitVec(I.vec, I.maxlength);
-  fread (&I.maxlength, soi, 3, fp ); 
+  fread ((char *)&I.maxlength, soi, 3, fp ); 
   I.maxlength = I.length;
   I.vec = (DigitType *) newDigitVec (&I.maxlength);
-  fread (I.vec , soD, I.length, fp);
+  fread ((char *)I.vec , soD, I.length, fp);
 }
 
 void bigint::write_to_file(FILE * fp)
@@ -772,8 +860,8 @@ void bigint::write_to_file(FILE * fp)
   int soi, soD;
   soi = sizeof(int);
   soD = sizeof(DigitType);
-  fwrite (&I.maxlength, soi, 3, fp); 
-  fwrite (I.vec, soD, I.length, fp); 
+  fwrite ((char *)&I.maxlength, soi, 3, fp); 
+  fwrite ((char *)I.vec, soD, I.length, fp); 
 }
 
 /**
