@@ -5,6 +5,18 @@
 // File        : bigint.c (implementation of the lip interface) 
 // Author      : Thomas Papanikolaou (TP)
 // Last change : TP, Feb 7 1995, initial version
+//               AH, Sep 25 1996,
+//                   Changed the return-value type of all 
+//                   yes/no-functions to bool. 
+//               MM, Sep 25 1996
+//                   Changed the return-value type of the
+//                   postfix operators from bigint & to bigint
+//                   because the return-value must be a local variable.
+//		 MM, Sep 25 1996
+//		     Modified power-functions to handle
+//		     negative exponents.
+//               MM, Oct 21 1996
+//                   Added test for 0 in randomize.
 //
 
 #include <LiDIA/bigint.h>
@@ -40,7 +52,7 @@ bigint::bigint(unsigned long ul)
     *I = 1;
 }
 
-bigint::bigint(double d)
+void bigint::assign(double d)
 
 {
   double mantisse;
@@ -95,7 +107,7 @@ bigint::~bigint()
 int bigint::bit(unsigned int i) const
 { return zbit(I, (long)i); }
 
-int bigint::length() const
+lidia_size_t bigint::length() const
 {
   long l = 0;
   if (I != 0)
@@ -107,56 +119,66 @@ int bigint::length() const
   return (int)l;
 }
 
-int bigint::bit_length() const
+lidia_size_t bigint::bit_length() const
 { return z2log(I); } 
 
 int bigint::sign() const
 { return zsign(I); } 
 
-int bigint::is_positive() const
-{ return (zsign(I) > 0); } 
+bool bigint::is_positive() const
+{ if (zsign(I) > 0) return true;
+  return false; } 
 
-int bigint::is_negative() const
-{ return (zsign(I) < 0); }
+bool bigint::is_negative() const
+{ if (zsign(I) < 0) return true;
+  return false; }
 
-int bigint::is_even() const
-{ return !zodd(I); }
+bool bigint::is_even() const
+{ if (!zodd(I)) return true;
+  return false; }
 
-int bigint::is_odd() const
-{ return zodd(I); }
+bool bigint::is_odd() const
+{ if (zodd(I)) return true;
+  return false; }
 
-int bigint::is_zero() const
-{ return (zsign(I) == 0); }
+bool bigint::is_zero() const
+{ if (zsign(I) == 0) return true;
+  return false; }
 
-int bigint::is_gt_zero() const
-{ return (zsign(I) > 0); }
+bool bigint::is_gt_zero() const
+{ if (zsign(I) > 0) return true;
+  return false; }
 
-int bigint::is_ge_zero() const
-{ return (zsign(I) >= 0); }
+bool bigint::is_ge_zero() const
+{ if (zsign(I) >= 0) return true;
+  return false; }
 
-int bigint::is_lt_zero() const
-{ return (zsign(I) < 0); }
+bool bigint::is_lt_zero() const
+{ if (zsign(I) < 0) return true;
+  return false; }
 
-int bigint::is_le_zero() const
-{ return (zsign(I) <= 0); }
+bool bigint::is_le_zero() const
+{ if (zsign(I) <= 0) return true;
+  return false; }
 
-int bigint::is_one() const
-{ return ((I != 0) && (*I == 1) && (I[1] == 1)); } 
+bool bigint::is_one() const
+{ if ((I != 0) && (*I == 1) && (I[1] == 1)) return true;
+  return false; } 
 
-int bigint::intify(int & i) const
+bool bigint::intify(int & i) const
 {
   if (z2log(I) > (CHARL * sizeof(int)-1))
-    return 1;
+    return true;
   i = (int) ztoint(I);
-    return 0;
+    return false;
 }
 
-int bigint::longify(long & i) const
+bool bigint::longify(long & i) const
 {
   if (z2log(I) > (CHARL * SIZEOFLONG-1))
-    return 1;
+    return true;
   i = ztoint(I);
-    return 0;
+    return false;
 }
 
 int bigint::abs_compare(const bigint & a) const
@@ -213,6 +235,9 @@ const int bigint::bits_per_digit()
 void bigint::absolute_value()
 { zabs(&I); }
 
+void bigint::abs()
+{ zabs(&I); }
+
 void bigint::negate()
 { znegate(&I); }
 
@@ -256,29 +281,37 @@ void bigint::divide_by_2()
 ** Type checking
 **/
 
-int is_char(const bigint & a)
-{ return (z2log(a.I) <= (CHARL-1)); }
+bool is_char(const bigint & a)
+{ if (z2log(a.I) <= (CHARL-1)) return true;
+  return false; }
 
-int is_uchar(const bigint & a)
-{ return (z2log(a.I) <= CHARL); }
+bool is_uchar(const bigint & a)
+{ if (z2log(a.I) <= CHARL) return true;
+  return false; }
 
-int is_short(const bigint & a)
-{ return (z2log(a.I) <= (CHARL * sizeof(short)-1)); }
+bool is_short(const bigint & a)
+{ if (z2log(a.I) <= (CHARL * sizeof(short)-1)) return true;
+  return false; }
 
-int is_ushort(const bigint & a)
-{ return (z2log(a.I) <= (CHARL * sizeof(unsigned short))); }
+bool is_ushort(const bigint & a)
+{ if (z2log(a.I) <= (CHARL * sizeof(unsigned short))) return true;
+  return false; }
+ 
+bool is_int(const bigint & a)
+{ if (z2log(a.I) <= (CHARL * sizeof(int)-1)) return true;
+  return false; }
 
-int is_int(const bigint & a)
-{ return (z2log(a.I) <= (CHARL * sizeof(int)-1)); }
+bool is_uint(const bigint & a)
+{ if (z2log(a.I) <= CHARL * sizeof(unsigned int)) return true;
+  return false; }
 
-int is_uint(const bigint & a)
-{ return (z2log(a.I) <= CHARL * sizeof(unsigned int)); }
+bool is_long(const bigint & a)
+{ if (z2log(a.I) <= (CHARL * sizeof(long)-1)) return true;
+  return false; }
 
-int is_long(const bigint & a)
-{ return (z2log(a.I) <= (CHARL * sizeof(long)-1)); }
-
-int is_ulong(const bigint & a)
-{ return (z2log(a.I) <= CHARL * sizeof(unsigned long)); }
+bool is_ulong(const bigint & a)
+{ if (z2log(a.I) <= CHARL * sizeof(unsigned long)) return true;
+  return false; }
 
 /**
 ** assignments
@@ -357,23 +390,23 @@ bigint & bigint::operator = (const bigint & a)
 ** comparisons
 **/
 
-int operator == (const bigint & a, const bigint & b)
-{ return zcompare(a.I, b.I) == 0;}
+bool operator == (const bigint & a, const bigint & b)
+{ if (zcompare(a.I, b.I) == 0) return true; else return false; }
 
-int operator != (const bigint & a, const bigint & b)
-{ return zcompare(a.I, b.I) != 0;}
+bool operator != (const bigint & a, const bigint & b)
+{ if (zcompare(a.I, b.I) != 0) return true; else return false; }
 
-int operator > (const bigint & a, const bigint & b)
-{ return zcompare(a.I, b.I) > 0;}
+bool operator > (const bigint & a, const bigint & b)
+{ if (zcompare(a.I, b.I) > 0) return true; else return false; }
 
-int operator >= (const bigint & a, const bigint & b)
-{ return zcompare(a.I, b.I) >= 0;}
+bool operator >= (const bigint & a, const bigint & b)
+{ if (zcompare(a.I, b.I) >= 0) return true; else return false; }
 
-int operator < (const bigint & a, const bigint & b)
-{ return zcompare(a.I, b.I) < 0;}
+bool operator < (const bigint & a, const bigint & b)
+{ if (zcompare(a.I, b.I) < 0) return true; else return false; }
 
-int operator <= (const bigint & a, const bigint & b)
-{ return zcompare(a.I, b.I) <= 0;}
+bool operator <= (const bigint & a, const bigint & b)
+{ if (zcompare(a.I, b.I) <= 0) return true; else return false; }
 
 /**
 ** operator overloading
@@ -472,11 +505,11 @@ bigint & bigint::operator++ ()
 bigint & bigint::operator-- ()
 { zsadd(I, -1, &I); return *this; }
 
-bigint & bigint::operator++ (int)
-{ zsadd(I, 1, &I); return *this; }
+bigint bigint::operator++ (int)
+{ bigint a = *this; zsadd(I, 1, &I); return a; }
 
-bigint & bigint::operator-- (int)
-{ zsadd(I, -1, &I); return *this; }
+bigint bigint::operator-- (int)
+{ bigint a = *this; zsadd(I, -1, &I); return a; }
 
 int bigint::operator ! () const
 { return ziszero(I); }
@@ -572,7 +605,27 @@ void power(bigint & c, const bigint & a, const bigint & b)
 {
   static verylong tmp1 = 0;
 
-  if (zsign(a.I) < 0)
+  if (zsign(b.I) == 0)   // b == 0 -> c = 1
+     zone(&c.I);
+  else if (zsign(b.I) < 0)
+   {
+     if ((a.I)[1] == 1)
+      {
+        if ((a.I)[0] == 1) // a == 1
+      	   zone(&c.I);
+        else if ((a.I)[0] == -1) // a == -1
+ 	 {
+           zone(&c.I);
+           int e = (int) zodd(b.I);
+           if (e)  znegate(&c.I);
+	 }
+        else
+           zzero(&c.I);
+      }
+     else
+        zzero(&c.I);
+   }
+  else  if (zsign(a.I) < 0)
   {
     int e = (int) zodd(b.I);
     zcopy(a.I, &tmp1);
@@ -589,7 +642,26 @@ void power(bigint & c, const bigint & a, long i)
 {
   static verylong tmp1 = 0;
 
-  if (zsign(a.I) < 0)
+  if (i == 0)	// i == 0 -> c = 1
+     zone(&c.I);
+  else if (i < 0) 
+   {
+     if ((a.I)[1] == 1)
+      {
+        if ((a.I)[0] == 1) // a == 1
+           zone(&c.I);
+        else if ((a.I)[0] == -1) // a == -1
+         {
+           zone(&c.I);
+           if (i & 1) znegate(&c.I);
+         }
+        else
+           zzero(&c.I);
+      }
+     else
+        zzero(&c.I);
+   }
+  else if (zsign(a.I) < 0)
   {
     zcopy(a.I, &tmp1);
     zabs(&tmp1);
@@ -703,11 +775,53 @@ void seed(const bigint & a)
 { zrstart(a.I); }
 
 bigint randomize(const bigint & a)
-{ bigint c; zrandomb(a.I, &c.I); return c; }
+{
+  bigint c;
+
+  if (a.is_zero())
+     lidia_error_handler("bigint", "Bound must not be equal to zero.");
+  else
+   {
+     if (a.is_gt_zero())
+        zrandomb(a.I, &c.I);
+     else
+      {
+        bigint d;
+	negate(d,a);
+	zrandomb(d.I,&c.I);
+	c.negate();		
+      }
+   }
+
+  return c;
+ }
 
 double dbl(const bigint & a)
 { return zdoub(a.I); }
 
+xdouble xdbl(const bigint & a)
+{
+  xdouble d = 0.0;
+  if (zsign(a.I) != 0)
+  {
+    int l = a.I[0];
+    int i = 2;
+    if (l < 0) l = -l;
+    d = (double )a.I[1]; 
+    xdouble base = bigint::radix();
+    xdouble power = base;
+    while (i <= l)
+    {
+     d += a.I[i] * power;
+     power *= base;
+     i++;
+    }
+    if (a.I[0] < 0)
+      d = -d;
+  }
+  return d;
+}
+
 void sqrt(bigint & a, const bigint & b)
 { zroot(b.I, 2, &a.I); }
       
@@ -782,8 +896,8 @@ ostream & operator << (ostream & out, const bigint & a)
   return out;
 }
 
-int string_to_bigint(char *s, bigint & a)
-{ return zstrtoz(s, &a.I); }
+int string_to_bigint(const char *s, bigint & a)
+{ return zstrtoz((char*)s, &a.I); }
 
 
 int bigint_to_string(const bigint & n, char *s)
@@ -861,3 +975,13 @@ void bigint::scan_from_file(FILE * fp)
 void bigint::print_to_file(FILE * fp)
 { zfwrite(fp, I); }
 
+
+
+
+
+
+
+
+
+
+
