@@ -5,6 +5,7 @@
 // File        : newton_root.c 
 // Author      : Thomas Papanikolaou (TP)
 // Last change : TP, Feb 7 1995, initial version
+//               VM, Jan 6 1997, added break statements
 //
 
 #include <LiDIA/bigint.h>
@@ -20,10 +21,13 @@ newton_root(bigint & b, const bigint & a, int n)
   {
   case 0:
     b.assign_one();
+    break;
   case 1:
     b.assign(a);
+    break;
   case 2:
     sqrt(b, a);
+    break;
   default:
     b.assign_one();
     shift_left(b, b, (unsigned int)((a.bit_length() + n - 1) / n));
@@ -38,5 +42,6 @@ newton_root(bigint & b, const bigint & a, int n)
     power(c, b, n);
     if (c.compare(a) > 0)
       dec(b);
+    break;
   }
 }
