@@ -14,48 +14,52 @@ int alg_number::count = 0;
 #endif
 
 // Constructors & destructor:
-alg_number::alg_number(const order * O1):
-    den(1), coeff(O1->degree(),O1->degree()), O(O1) // Initialised with 0
+alg_number::alg_number(const nf_base * O1):
+    den(1), coeff(O1->degree(),O1->degree()), O((nf_base *)O1) // Initialised with 0
 {
-    debug_handler_c("alg_number","in alg_number(const order *)",1,
+    debug_handler_c("alg_number","in alg_number(const nf_base *)",1,
 		    count++;
 		    cout << "\nNow we have "<<count<<" alg_numbers!\n");
+    O->inc_ref();
 }
 
-alg_number::alg_number(const bigint & i, const order * O1):
-    den(1), coeff((O1->get_one()) * i), O(O1)
+alg_number::alg_number(const bigint & i, const nf_base * O1):
+    den(1), coeff((((nf_base *)O1)->get_one()) * i), O((nf_base *)O1)
 {
-    debug_handler_c("alg_number","in alg_number(const bigint &, order *)",
+    debug_handler_c("alg_number","in alg_number(const bigint &, nf_base *)",
 		    1,count++;
 		    cout << "\nNow we have "<<count<<" alg_numbers!\n");
+    O->inc_ref();
 }
 
 alg_number::alg_number(const base_vector <bigint> & a, 
-		       const bigint &i, const order * O1):
-    den(i), coeff(a), O(O1)
+		       const bigint &i, const nf_base * O1):
+    den(i), coeff(a), O((nf_base *)O1)
 
 {
     if (degree()!=a.size())
 	lidia_error_handler("alg_number","in alg_number "
 		      "(const base_vector <bigint> &, const bigint &, "
-		      "const order *)::base_vector of wrong size");
+		      "const nf_base *)::base_vector of wrong size");
     debug_handler_c("alg_number",
 		    "in alg_number"
-		    "(const base_vector &,const bigint &, const order *)",1,
+		    "(const base_vector &,const bigint &, const nf_base *)",1,
 		    count++;
 		    cout << "\nNow we have "<<count<<" alg_numbers!\n");
     normalize();
+    O->inc_ref();
 }
 
-alg_number::alg_number(const bigint * a, const bigint &i, const order * O1):
-    den(i), coeff(a, O1->degree()), O(O1)
+alg_number::alg_number(const bigint * a, const bigint &i, const nf_base * O1):
+    den(i), coeff(a, O1->degree()), O((nf_base *)O1)
 
 {
     debug_handler_c("alg_number",
-		    "in alg_number(bigint *, const bigint &,const order *)",
+		    "in alg_number(bigint *, const bigint &,const nf_base *)",
 		    1,count++;
 		    cout << "\nNow we have "<<count<<" alg_numbers!\n");
     normalize();
+    O->inc_ref();
 }
 
 alg_number::alg_number(const alg_number &a):
@@ -65,17 +69,41 @@ alg_number::alg_number(const alg_number &a):
     debug_handler_c("alg_number","in alg_number(const a_n &)",1,
 		    count++;
 		    cout << "\nNow we have "<<count<<" alg_numbers!\n");
+    O->inc_ref();
 }
 
 alg_number::~alg_number()
 {
-    debug_handler_c("alg_number","in ~alg_number()",1,
-		    count--;
-		    cout << "\nNow we have only "<<count<<" alg_numbers!\n");
+  debug_handler_c("alg_number","in ~alg_number()",1,
+		  count--;
+		  cout << "\nNow we have only "<<count<<" alg_numbers!\n");
+  O->dec_ref();
 }
 
 // member-functions
-bigfloat alg_number::get_conjugate(lidia_size_t i){}
+bigfloat alg_number::get_conjugate(lidia_size_t j){
+  if (j < 1 || j > degree())
+    lidia_error_handler("field","get_conjugate(lidia_size_t i, lidia_size_t j)"
+			"::j out of range (must be between 1 and n)");
+  math_vector <bigfloat> tmp1(degree(),degree()), tmp(degree(),degree());
+  ((nf_base *)O)->get_conjugates().row_vector(tmp1,--j);
+  for (register lidia_size_t i = 0; i< degree(); i++)
+    tmp[i].assign(coeff[i]);
+
+  bigfloat res;
+  multiply(res, tmp1, tmp);
+  divide(res, res, den);
+  return res;
+}
+
+math_vector <bigfloat> alg_number::get_conjugates(){
+  math_vector <bigfloat> tmp(degree(),degree());
+  for (register lidia_size_t i = 0; i< degree(); i++)
+    tmp[i].assign(coeff[i]);
+  multiply(tmp, ((nf_base *)O)->get_conjugates(), tmp);
+  divide(tmp, tmp, bigfloat(den));
+  return tmp;
+}
 
 bigint_matrix rep_matrix(const alg_number & a)
 // RepMatrix of a * denominator
@@ -83,7 +111,7 @@ bigint_matrix rep_matrix(const alg_number & a)
     debug_handler("alg_number","in function rep_matrix(const alg_number &)");
     bigint_matrix A(a.degree(), a.degree());	// initialised with 0 !
     // construct the matrix using either MT or the polynomial
-    const order * O = a.which_order();
+    nf_base * O = a.O;
     if (O->using_necessary()){
 	// If necessary construct MT
 	if (!(O->table_computed())){
@@ -117,7 +145,7 @@ bigint_matrix rep_matrix(const alg_number & a)
 	delete[] tmp;
 
 	for (register lidia_size_t i=0;i<a.degree();
-			       i++,p=p*x%(O->K->which_polynomial())){
+			       i++,p=p*x%(O->f)){
 	    register lidia_size_t l=0;
 	    for(; l <= p.degree(); l++)
 		A.sto(l,i,p[l]);
@@ -153,7 +181,7 @@ void alg_number::normalize()
     if (!d.is_one()){
 	den /= d;
 	for(i=0; i< degree(); i++)
-	    coeff.member(i) = coeff.member(i) / d;
+	    coeff[i] /= d;
     }
 }
 
@@ -190,29 +218,33 @@ void alg_number::invert()
 void alg_number::assign_zero()
 {
     for (register lidia_size_t i=0;i<degree();i++)
-	coeff.member(i) = 0;
+	coeff[i].assign_zero();
     den = 1;
 }
 
 void alg_number::assign_one()
 {
     debug_handler("alg_number","in member - function assign_one()");
-    coeff = O->get_one();
+    coeff.assign(O->get_one());
     den = 1;
 }
 
 void alg_number::assign(const bigint & a)
 {
-    coeff = (O->get_one()) * a;
+    multiply(coeff, O->get_one(), a);
     den = 1;
 }
 
 void alg_number::assign(const alg_number & a)
 {
-    debug_handler("alg_number","in member - function assign(a_n &)");
+  debug_handler("alg_number","in member - function assign(a_n &)");
+  if (O != a.O){
+    O->dec_ref();
     O = a.O;
-    coeff = a.coeff;
-    den = a.den;
+    O->inc_ref();
+  }
+  coeff = a.coeff;
+  den = a.den;
 }
 
 // Procedural versions:
@@ -222,7 +254,11 @@ void add(alg_number &c, const alg_number &a, const alg_number &b)
 	lidia_error_handler("alg_number","add(...)::addition of numbers from "
 		      "different orders is not yet implemented");
     }
-    c.O = a.which_order();
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
     bigint d = gcd(a.den, b.den);
     bigint e = a.den / d;
     
@@ -234,10 +270,14 @@ void add(alg_number &c, const alg_number &a, const alg_number &b)
 void subtract(alg_number &c, const alg_number &a, const alg_number &b)
 {
     if (a.O != b.O) {
-	lidia_error_handler("alg_number","subtract(...)::subtraction of numbers "
-		      "from different orders is not yet implemented");
+      lidia_error_handler("alg_number","subtract(...)::subtraction of numbers "
+			  "from different orders is not yet implemented");
+    }
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
     }
-    c.O = a.which_order();
     bigint d = gcd(a.den, b.den);
     bigint e = a.den / d;
 
@@ -253,8 +293,12 @@ void multiply(alg_number &c, const alg_number &a, const alg_number &b)
 	lidia_error_handler("alg_number","multiply(...)::multiplication of numbers "
 		      "from different orders is not yet implemented");
     }
-    c.O = a.which_order();
-    const order * O = a.which_order();
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    nf_base * O = a.O;
     c.den = a.den * b.den;
     if (O->using_necessary()){
 	// If necessary construct MT
@@ -263,16 +307,16 @@ void multiply(alg_number &c, const alg_number &a, const alg_number &b)
 	}
 	if ((&c != &a) && (&c != &b))
 	    for(register lidia_size_t k=0; k<a.degree(); k++){
-		c.coeff.member(k)=0;
+		c.coeff[k].assign_zero();
 		bigint * MTpoint=O->table.column(k);
 		bigint * TMPpoint = MTpoint;
 		for(register lidia_size_t i=0; i<a.degree(); i++){
 		    for(register lidia_size_t j=0; j<i; j++, MTpoint++){
-			c.coeff.member(k) += *MTpoint * a.coeff.member(i) 
+			c.coeff[k] += *MTpoint * a.coeff.member(i) 
 			    * b.coeff.member(j)
 			    + *MTpoint * a.coeff.member(j) * b.coeff.member(i);
 		    }
-		    c.coeff.member(k) += *MTpoint * a.coeff.member(i)
+		    c.coeff[k] += *MTpoint * a.coeff.member(i)
 			* b.coeff.member(i);
 		    MTpoint++;
 		}
@@ -281,15 +325,15 @@ void multiply(alg_number &c, const alg_number &a, const alg_number &b)
 	else{
 	    math_vector <bigint> tmp(a.degree(),a.degree());
 	    for(register lidia_size_t k = 0; k < a.degree(); k++){
-		tmp.member(k) = 0;
+		tmp[k] = 0;
 		bigint * MTpoint=O->table.column(k);
 		bigint * TMPpoint = MTpoint;
 		for(register lidia_size_t i=0; i<a.degree(); i++){
 		    for(register lidia_size_t j=0; j<i; j++, MTpoint++)
-			tmp.member(k) += *MTpoint * a.coeff.member(i) 
+			tmp[k] += *MTpoint * a.coeff.member(i) 
 			    * b.coeff.member(j)
 			    + *MTpoint * a.coeff.member(j) * b.coeff.member(i);
-		    tmp.member(k) += *MTpoint * a.coeff.member(i)
+		    tmp[k] += *MTpoint * a.coeff.member(i)
 			* b.coeff.member(i);
 		    MTpoint++;
 		}
@@ -307,7 +351,7 @@ void multiply(alg_number &c, const alg_number &a, const alg_number &b)
 	polynomial <bigint> pb(tmp = b.coeff.get_data(),a.degree()-1);
 	delete[] tmp;
 
-	polynomial <bigint> f = pa*pb%((O->K)->which_polynomial());
+	polynomial <bigint> f = pa*pb%(O->f);
 	register lidia_size_t l=0;
 	for(; l <= f.degree(); l++)
 	    c.coeff[l] = f[l];
@@ -336,7 +380,11 @@ void divide(alg_number &c, const alg_number &a, const alg_number &b)
     tmp2.reginvimage(A,tmp);
 
     // initialise result with solution
-    c.O = a.which_order();
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
     c.coeff.set_data(temp=tmp2.column(0),a.degree());
         
     // get denominator
@@ -353,10 +401,12 @@ void divide(alg_number &c, const alg_number &a, const alg_number &b)
 
 void add(alg_number &c, const alg_number &a, const bigint & b)
 {
-    const order * O = a.which_order();
-    alg_number to_add((O->get_one()) * b, 1, O);
-    
-    c.O = O;
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    alg_number to_add((c.O->get_one()) * b, 1, c.O);
     c.coeff = ((a.coeff) + ((to_add.coeff) * a.den));
     c.den = a.den;
     c.normalize();
@@ -364,10 +414,12 @@ void add(alg_number &c, const alg_number &a, const bigint & b)
 
 void subtract(alg_number &c, const alg_number &a, const bigint & b)
 {
-    const order * O = a.which_order();
-    alg_number to_sub((O->get_one()) * b, 1, O);
-    
-    c.O = O;
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    alg_number to_sub((c.O->get_one()) * b, 1, c.O);
     c.coeff = ((a.coeff) - ((to_sub.coeff) * a.den));
     c.den = a.den;
     c.normalize();
@@ -377,7 +429,11 @@ void multiply(alg_number &c, const alg_number &a, const bigint & b)
 {
     bigint d = gcd(a.den, b);
 
-    c.O = a.which_order();
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
     c.den = a.den / d;
     c.coeff = a.coeff * (b / d);
 }
@@ -387,7 +443,11 @@ void divide(alg_number &c, const alg_number &a, const bigint & b)
     debug_handler("alg_number",
 		  "in function divide(a_n &, const a_n &, const bigint &)");
 
-    c.O = a.which_order();
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
     c.coeff = a.coeff;
 
     if (&c != &a){
@@ -405,10 +465,12 @@ void divide(alg_number &c, const alg_number &a, const bigint & b)
 
 void add(alg_number &c, const bigint & b, const alg_number &a)
 {
-    const order * O = a.which_order();
-    alg_number to_add((O->get_one()) * b, 1, O);
-    
-    c.O = O;
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    alg_number to_add((c.O->get_one()) * b, 1, c.O);
     c.coeff = ((a.coeff) + ((to_add.coeff) * a.den));
     c.den = a.den;
     c.normalize();
@@ -416,10 +478,12 @@ void add(alg_number &c, const bigint & b, const alg_number &a)
 
 void subtract(alg_number &c, const bigint & b, const alg_number &a)
 {
-    const order * O = a.which_order();
-    alg_number sub((O->get_one()) * b, 1, O);
-    
-    c.O = O;
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    alg_number sub((c.O->get_one()) * b, 1, c.O);
     c.coeff = (((sub.coeff) * a.den) - a.coeff);
     c.den = a.den;
     c.normalize();
@@ -429,7 +493,11 @@ void multiply(alg_number &c, const bigint &b, const alg_number &a)
 {
     bigint d = gcd(a.den, b);
 
-    c.O = a.which_order();
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
     c.den = a.den / d;
     c.coeff = a.coeff * (b / d);
 }
@@ -438,7 +506,7 @@ void divide(alg_number &c, const bigint &b, const alg_number &a)
 {
     debug_handler("alg_number",
 		  "in function divide(a_n &, const bigint &, const a_n &)");
-    const order * O = a.which_order();
+    nf_base * O = a.O;
     alg_number dividend(O->get_one() * b, 1, O);
 
     divide(c,dividend,a);
@@ -449,10 +517,13 @@ void power(alg_number &c, const alg_number &a, const bigint & b)
     debug_handler("alg_number",
 		  "in function power(a_n &, const a_n &, const bigint &)");
     bigint exponent;
-    const order * O = a.which_order();
-    alg_number multiplier(O);
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    alg_number multiplier(c.O);
     
-    c.O=O;
     if (b.is_negative())
 	power(c, inverse(a), -b);
     else if (b.is_zero() || a.is_one())
@@ -489,10 +560,13 @@ void power_mod_p(alg_number &c, const alg_number &a, const bigint & b,
 		  "in function power(a_n &, const a_n &, const bigint &,"
 		  "const bigint &)");
     bigint exponent;
-    const order * O = a.which_order();
-    alg_number multiplier(O);
-    
-    c.O=O;
+    if (c.O != a.O){
+      c.O->dec_ref();
+      c.O = a.O;
+      c.O->inc_ref();
+    }
+    alg_number multiplier(c.O);
+
     if (b.is_negative())
 	power_mod_p(c, inverse(a), -b, p);
     else if (b.is_zero() || a.is_one())
@@ -500,6 +574,8 @@ void power_mod_p(alg_number &c, const alg_number &a, const bigint & b,
     else{	
 	exponent.assign(b);
 	multiplier.assign(a);
+	for(register lidia_size_t i = 0; i < multiplier.coeff.size(); i++)
+	  multiplier.coeff[i] %= p;
 	c.assign_one();
 	while (exponent.is_gt_zero()){
 	    if (!exponent.is_even()){
@@ -543,8 +619,7 @@ bigrational norm(const alg_number & a)
 {
     bigint normdenominator;
     power(normdenominator, a.den, bigint(a.degree()));
-    bigrational norm (rep_matrix(a).det(),normdenominator);
-    return norm;
+    return bigrational(rep_matrix(a).det(),normdenominator);
 }
 
 bigrational trace(const alg_number & a)
@@ -553,8 +628,7 @@ bigrational trace(const alg_number & a)
     bigint_matrix A(rep_matrix(a));
     bigint tmp;
     A.trace(tmp);
-    bigrational res (tmp, a.den);
-    return res;
+    return bigrational(tmp, a.den);
 }
 
 polynomial <bigint> charpoly(const alg_number & a)
@@ -576,8 +650,7 @@ polynomial <bigint> charpoly(const alg_number & a)
 // Other functions:
 void negate(alg_number & b, const alg_number & a)
 {
-    alg_number c(-a.coeff, a.den, a.which_order());
-    b=c;
+    b.assign(alg_number(-a.coeff, a.den, a.O));
 }
 
 void multiply_by_2(alg_number & c, const alg_number & a)
@@ -594,7 +667,7 @@ void divide_by_2(alg_number & c, const alg_number & a)
 
 void invert(alg_number &c, const alg_number & a)
 {
-    c = a;
+    c.assign(a);
     c.invert();
 }
 
@@ -615,11 +688,13 @@ void square(alg_number & a, const alg_number & b)
 void swap(alg_number & a, alg_number & b)
 {
     swap (a.den, b.den);
-    math_vector <bigint> help = a.coeff;
-    a.coeff = b.coeff;
-    b.coeff = help;
+//     math_vector <bigint> help = a.coeff;
+//     a.coeff = b.coeff;
+//     b.coeff = help;
 
-    const order * O = a.O;
+    swap(a.coeff, b.coeff);
+
+    nf_base * O = a.O;
     a.O = b.O;
     b.O = O;
 }
@@ -643,20 +718,14 @@ ostream& operator <<(ostream & s, const alg_number & a)
 
 istream& operator >>(istream & s, alg_number & a)
 {
-  if (current_order == NULL){
-    if (current_number_field == NULL)
-      lidia_error_handler ("alg_number",
-			   "operator >>::No number_field or order!");
-    else{
-      order * dummy_order = new order(current_number_field);
-      // this may lead to memory-problems, but since you typically
-      // shouldn't need this feature very often, it's not worth 
-      // the trouble doing this cleaner...
-      a.O = current_order = dummy_order;
-    }
+  if (nf_base::current_base == nf_base::dummy_base)
+    lidia_error_handler ("alg_number",
+			 "operator >>::No number_field or order!");
+  if (a.O != nf_base::current_base){
+    a.O->dec_ref();
+    a.O = nf_base::current_base;
+    a.O->inc_ref();
   }
-  else
-    a.O = current_order;
   s >> a.coeff;
   char c;
   do{
