@@ -6,7 +6,13 @@
 // Author      : Thomas Papanikolaou (TP)
 // Last change : TP, Feb 7 1995, initial version
 //               TP, Oct 30 1995, alphabetical order
-//
+//		 WB, Oct 15 1996, added
+//			bigfloat(xdouble xd);
+//		 	int xdoublify(xdouble & d) const;
+//			void assign(xdouble xd);
+//			xdouble operator = (xdouble d);
+//			
+
 
 #include <LiDIA/bigfloat.h>
 #include <LiDIA/bigfloat_config.h>
@@ -494,6 +500,17 @@ void bigfloat::assign(double d)
 }
 #endif
 
+void bigfloat::assign(xdouble xd)
+{
+  bigfloat temp(xd.l());
+  assign(xd.h());
+  add(*this, *this, temp);
+//   char s[200];
+//   xdouble_to_string(s, xd);
+//   string_to_bigfloat(s, *this);
+}
+
+
 void bigfloat::assign(int i)
 {
   m.assign(i);
@@ -578,6 +595,12 @@ double bigfloat::operator = (double d)
   this->assign(d);
   return d;
 }
+
+xdouble bigfloat::operator = (xdouble xd)
+{
+  this->assign(xd);
+  return xd;
+}
  
 bigint bigfloat::operator = (const bigint & bi)
 {
@@ -949,160 +972,177 @@ void ceil(bigint & y, const bigfloat & x)
     inc(y);
 }
 
-int is_char(const bigfloat & x)
+bool is_char(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(char)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_uchar(const bigfloat & x)
+bool is_uchar(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(unsigned char
 )));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_short(const bigfloat & x)
+bool is_short(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(short)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_ushort(const bigfloat & x)
+bool is_ushort(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(unsigned short)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_int(const bigfloat & x)
+bool is_int(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(int)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_uint(const bigfloat & x)
+bool is_uint(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(unsigned int)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_long(const bigfloat & x)
+bool is_long(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(long)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_ulong(const bigfloat & x)
+bool is_ulong(const bigfloat & x)
 {
 
   if (x.is_zero())
-    return 1;
+    return true;
 
   bigfloat tmp;
   truncate(tmp, x);
   int b = (tmp.compare(x) == 0 && (tmp.prec <= bits_per_(unsigned long)));
 
-  return b;
+  if (b) return true;
+  return false;
 }
 
-int is_double(const bigfloat & x)
+bool is_double(const bigfloat & x)
 {
   long ex = x.prec;
   if (ex == 0)
-    return 1;
+    return true;
   ex += x.e;
-  return ((ex <= 0x3ff) && (ex > -1023));
+  if ((ex <= 0x3ff) && (ex > -1023)) return true;
+  return false;
 }
 
-int bigfloat::is_zero() const
+bool bigfloat::is_zero() const
 {
-  return (m.is_zero() || this->is_pm_zero());
+  if (m.is_zero() || this->is_pm_zero()) return true;
+  return false;
 }
  
-int bigfloat::is_exact_zero() const
+bool bigfloat::is_exact_zero() const
 {
-  return (m.is_zero() && this->is_exact());
+  if (m.is_zero() && this->is_exact()) return true;
+  return false;
 }
  
-int bigfloat::is_approx_zero() const
+bool bigfloat::is_approx_zero() const
 {
-  return (m.is_zero() || ((e + prec) < (-bigfloat::binary_precision + rounding_bits)));
+  if (m.is_zero() || ((e + prec) < (-bigfloat::binary_precision + rounding_bits))) return true;
+return false;
 }
  
-int bigfloat::is_gt_zero() const
+bool bigfloat::is_gt_zero() const
 {
-  return m.is_gt_zero();
+  if (m.is_gt_zero()) return true;
+  return false;
 }
  
-int bigfloat::is_lt_zero() const
+bool bigfloat::is_lt_zero() const
 {
-  return m.is_lt_zero();
+  if (m.is_lt_zero()) return true;
+  return false;
 }
  
-int bigfloat::is_ge_zero() const
+bool bigfloat::is_ge_zero() const
 {
-  return m.is_ge_zero();
+  if (m.is_ge_zero()) return true;
+  return false;
 }
  
-int bigfloat::is_le_zero() const
+bool bigfloat::is_le_zero() const
 {
-  return m.is_le_zero();
+  if (m.is_le_zero()) return true;
+  return false;
 }
  
-int bigfloat::is_one() const
+bool bigfloat::is_one() const
 {
   bigfloat tmp(1);
   subtract(tmp, tmp, *this);
-  return tmp.is_zero();
+  if  (tmp.is_zero()) return true;
+  return false;
 }
  
 int bigfloat::abs_compare(const bigfloat & y) const
@@ -1126,11 +1166,11 @@ int bigfloat::compare(const bigfloat & y) const
   return gt;
 }
 
-int operator == (const bigfloat & x, const bigfloat & y)
+bool operator == (const bigfloat & x, const bigfloat & y)
 {
 
   if (x.m.sign() != y.m.sign())
-    return 0;
+    return false;
 
   if (x.m.is_zero())
     return y.m.is_zero();
@@ -1139,18 +1179,18 @@ int operator == (const bigfloat & x, const bigfloat & y)
     return x.m.is_zero();
 
   if ((x.e + x.m.bit_length()) != (y.e + y.m.bit_length()))
-    return 0;
+    return false;
 
   bigfloat tmp;
   subtract(tmp, x, y);
   return (tmp.is_zero());
 }
 
-int operator != (const bigfloat & x, const bigfloat & y)
+bool operator != (const bigfloat & x, const bigfloat & y)
 {
 
   if (x.m.sign() != y.m.sign())
-    return 1;
+    return true;
 
   if (x.m.is_zero())
     return !y.m.is_zero();
@@ -1159,14 +1199,14 @@ int operator != (const bigfloat & x, const bigfloat & y)
     return !x.m.is_zero();
 
   if ((x.e + x.m.bit_length()) != (y.e + y.m.bit_length()))
-    return 1;
+    return true;
 
   bigfloat tmp;
   subtract(tmp, x, y);
   return (!tmp.is_zero());
 }
 
-int operator > (const bigfloat & x, const bigfloat & y)
+bool operator > (const bigfloat & x, const bigfloat & y)
 {
   long sx, sy, ex, ey;
 
@@ -1176,24 +1216,30 @@ int operator > (const bigfloat & x, const bigfloat & y)
   // if (x == 0) return (y < 0)
   
   if (sx == 0)
-    return (sy < 0);
+    {
+      if (sy < 0) return true;
+      return false;
+    }
   
   // if (y == 0) return (x > 0)
   
   if (sy == 0)
-    return (sx > 0);
-  
+    {
+      if (sx > 0) return true;
+      return false;
+    }
+
   // at this point is x != 0 && y != 0
   
   // if (x > 0) && (y < 0) return 1;
   
   if (sx > sy)
-    return 1;
+    return true;
   
   // if (x < 0) && (y > 0) return 0;
   
   if (sx < sy)
-    return 0;
+    return false;
   
   // at this point sx == sy
   
@@ -1203,13 +1249,19 @@ int operator > (const bigfloat & x, const bigfloat & y)
   // if (ex > ey) return (sx > 0);
   
   if (ex > ey)
-    return (sx > 0);
-  
+    {
+      if (sx > 0) return true;
+      return false;
+    }
+
   // if (ex < ey) return (sx < 0);
   
   if (ex < ey)
-    return (sx < 0);
-  
+    {
+      if (sx < 0) return true;
+      return false;
+    }
+
   // at this point ex == ey
   
   // return ( (x-y) > 0)
@@ -1221,14 +1273,15 @@ int operator > (const bigfloat & x, const bigfloat & y)
   
 }
 
-int operator >= (const bigfloat & x, const bigfloat & y)
+bool operator >= (const bigfloat & x, const bigfloat & y)
 {
   int gt = (x > y);
   int eq = (x == y);
-  return (gt || eq);
+  if (gt || eq) return true;
+  return false;
 }
 
-int operator < (const bigfloat & x, const bigfloat & y)
+bool operator < (const bigfloat & x, const bigfloat & y)
 {
   long sx, sy, ex, ey;
   
@@ -1238,24 +1291,30 @@ int operator < (const bigfloat & x, const bigfloat & y)
   // if (x == 0) return (y > 0)
   
   if (sx == 0)
-    return (sy > 0);
-  
+    {
+      if (sy > 0) return true;
+      return false;
+    }
+
   // if (y == 0) return (x < 0)
   
   if (sy == 0)
-    return (sx < 0);
-  
+    {
+      if (sx < 0) return true;
+      return false;
+    }
+
   // at this point is x != 0 && y != 0
   
   // if (x > 0) && (y < 0) return 0;
   
   if (sx > sy)
-    return 0;
+    return false;
   
   // if (x < 0) && (y > 0) return 1;
   
   if (sx < sy)
-    return 1;
+    return true;
   
   // at this point sx == sy
   
@@ -1265,12 +1324,18 @@ int operator < (const bigfloat & x, const bigfloat & y)
   // if (ex > ey) return (sx < 0);
   
   if (ex > ey)
-    return (sx < 0);
+    {
+      if (sx < 0) return true;
+      return false;
+    }
   
   // if (ex < ey) return (sx > 0);
   
   if (ex < ey)
-    return (sx > 0);
+    {
+      if (sx > 0) return true;
+      return false;
+    }
   
   // at this point ex == ey
   
@@ -1283,14 +1348,15 @@ int operator < (const bigfloat & x, const bigfloat & y)
   
 }
 
-int operator <= (const bigfloat & x, const bigfloat & y)
+bool operator <= (const bigfloat & x, const bigfloat & y)
 {
   int lt = (x < y);
   int eq = (x == y);
-  return (lt || eq);
+  if (lt || eq) return true;
+  return false;
 }
 
-int bigfloat::operator ! ()
+bool bigfloat::operator ! ()
 {
   return this->is_zero();
 }
@@ -1320,6 +1386,11 @@ bigfloat::bigfloat(double d)
   this->assign(d);
 }
 
+bigfloat::bigfloat(xdouble xd)
+{
+  this->assign(xd);
+}
+
 bigfloat::bigfloat(const bigint & bi)
 {
   this->assign(bi);
@@ -1352,34 +1423,63 @@ void bigfloat::bigintify(bigint & y) const
   else shift_right(y, y, -r.e);
 }
 
-int bigfloat::doublify(double &d) const
+bool bigfloat::doublify(double &d) const
 {
   long l = m.bit_length(), expo = e + l;
 
   if (m.is_zero() || (expo < -1023))
     {
       d = 0.0;
-      return 0;
+      return false;
     }
 
   if (expo >= 0x3ff)
     {
       warning_handler("bigfloat", "impossible assignment double = bigfloat");
-      return 1;
+      return true;
     }
 
-  if (l > 1023)
+  if (l > (SIZEOF_DOUBLE * 8))
   {
-    long ed = l - 1023;
+    long ed = l - (SIZEOF_DOUBLE * 8);
     bigint bd = m >> ed;                     // bd = (m/2^ed)*2^ed
     d = dbl(bd) * ldexp(2.0, (int)(e + ed - 1));
   }
   else 
     d = dbl(m) * ldexp(2.0, (int)(e - 1));
-  return 0;
+  return false;
 }
 
-int bigfloat::longify(long &i) const
+bool bigfloat::xdoublify(xdouble &xd) const
+{
+  long l = m.bit_length(), expo = e + l;
+
+  if (m.is_zero() || (expo < -996))
+    {
+      xd = 0.0;
+      return false;
+    }
+
+  if (expo >= 996)
+    {
+      warning_handler("bigfloat", "impossible assignment xdouble = bigfloat"
+);
+      return true;
+    }
+
+  if (l > (2*SIZEOF_DOUBLE * 8))
+  {
+    long ed = l - (2*SIZEOF_DOUBLE * 8);
+    bigint bd = m >> ed;                     // bd = (m/2^ed)*2^ed
+    xd = xdbl(bd) * exp(log((xdouble )2.0) * (int)(e + ed));
+  }
+  else 
+    xd = xdbl(m) * exp(log((xdouble )2.0) * (int)(e));
+  return false;
+}
+
+
+bool bigfloat::longify(long &i) const
 {
   double d;
   int flag;
@@ -1387,13 +1487,13 @@ int bigfloat::longify(long &i) const
   if (flag == 1 || (d > max_long))
     {
       warning_handler("bigfloat", "impossible assignment long = bigfloat");
-      return 1;
+      return true;
     }
   i = (long) d;
-  return 0;
+  return false;
 }
 
-int bigfloat::intify(int &i) const
+bool bigfloat::intify(int &i) const
 {
   double d;
   int flag;
@@ -1402,10 +1502,10 @@ int bigfloat::intify(int &i) const
   if (flag == 1 || (d > max_int))
     {
       warning_handler("bigfloat", "impossible assignment int = bigfloat");
-      return 1;
+      return true;
     }
   i = (int) d;
-  return 0;
+  return false;
 }
 
 void cos(bigfloat & y, const bigfloat & x)
@@ -2916,6 +3016,22 @@ void bigfloat::precision(long t)
     }
 }
 
+void bigfloat::set_precision(long t)
+{
+  if (t > 0)
+    {
+      bigfloat::decimal_precision = t;
+      bigfloat::digit_precision = 3 + (long) (ceil((t / (L2B10 * bits_per_base_digit))));
+      bigfloat::binary_precision = bits_per_base_digit * bigfloat::digit_precision;
+    }
+  else
+    {
+      bigfloat::decimal_precision = (long) ceil(2*bits_per_base_digit * L2B10);
+      bigfloat::binary_precision = 5 * bits_per_base_digit;
+      bigfloat::digit_precision = 5;
+    }
+}
+
 long bigfloat::get_precision()
 {
   return bigfloat::decimal_precision;
@@ -3038,14 +3154,14 @@ bigfloat round(const bigfloat & x)
 void shift_left(bigfloat & res, const bigfloat & x, long i)
 {
   if (&res != &x)
-    res.m.assign(x.m);
+    res.assign(x);
   res.e = x.e + i;
 }
 
 void shift_right(bigfloat & res, const bigfloat & x, long i)
 {
   if (&res != &x)
-    res.m.assign(x.m);
+    res.assign(x);
   res.e = x.e - i;
 }
 
@@ -3264,7 +3380,7 @@ bigfloat sqrt(const bigfloat & x)
   return y;
 }
 
-int string_to_bigfloat(const char *s, bigfloat & n)
+int string_to_bigfloat(char *s, bigfloat & n)
 {
   int mi, ei, eii, count;
   int j, l = strlen(s), expo = 0;
@@ -3990,12 +4106,12 @@ bigint bigfloat::mantissa() const
   return m;
 }
  
-int bigfloat::length() const
+lidia_size_t bigfloat::length() const
 {
   return m.length();
 }
  
-int bigfloat::bit_length() const
+lidia_size_t bigfloat::bit_length() const
 {
   return prec;
 }
