@@ -6,8 +6,10 @@
 // Author      : Thomas Papanikolaou (TP)
 // Last change : TP, Jan 29 1995, initial version 
 //               VM, Feb 7 1995, modified operators +, -, *, /
-//               to use the procedures add(), subtract(), ...
-//               Added a space ' ' in the output routine.
+//                   to use the procedures add(), subtract(), ...
+//                   Added a space ' ' in the output routine.
+//		 MM, Sep 11 1996, added 
+//		     assign(const bigint&,const bigint&)
 //
 
 #include <LiDIA/bigrational.h>
@@ -87,7 +89,7 @@ bigrational::bigrational(const bigint & n)
 bigrational::bigrational(const bigint & n, const bigint & d)
 {
   if (d.is_zero())
-    lidia_error_handler("bigrational", "consrtuctor(n,d)::division by zero.");
+    lidia_error_handler("bigrational", "constructor(n,d)::division by zero.");
   num.assign(n);
   den.assign(d);
   this->normalize();
@@ -113,55 +115,63 @@ sign() const
   return num.sign();
 }
 
-int bigrational::
+bool bigrational::
 is_positive() const
 {
-  return (num.sign() > 0);
+  if (num.sign() > 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_negative() const
 {
-  return (num.sign() < 0);
+  if (num.sign() < 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_zero() const
 {
-  return (num.sign() == 0);
+  if (num.sign() == 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_gt_zero() const
 {
-  return (num.sign() > 0);
+  if (num.sign() > 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_ge_zero() const
 {
-  return (num.sign() >= 0);
+  if (num.sign() >= 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_lt_zero() const
 {
-  return (num.sign() < 0);
+  if (num.sign() < 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_le_zero() const
 {
-  return (num.sign() <= 0);
+  if (num.sign() <= 0) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 is_one() const
 {
-  return (num.is_one() && den.is_one());
+  if (num.is_one() && den.is_one()) return true;
+  return false;
 }
 
-int bigrational::
+bool bigrational::
 intify(int &i) const
 {
   bigint I;
@@ -169,7 +179,7 @@ intify(int &i) const
   return I.intify(i);
 }
 
-int bigrational::
+bool bigrational::
 longify(long &i) const
 {
   bigint I;
@@ -262,6 +272,17 @@ assign_one()
   den.assign_one();
 }
 
+void bigrational::
+assign(const bigint & n,
+       const bigint & d)
+{
+  if (d.is_zero())
+    lidia_error_handler("bigrational", "assign(n,d)::division by zero.");
+  num.assign(n);
+  den.assign(d);
+  this->normalize();
+}
+
 void bigrational::
 assign(const bigrational & a)
 {
@@ -291,7 +312,7 @@ divide_by_2()
 ** Type checking
 **/
 
-int
+bool
 is_bigint(const bigrational & a)
 {
   return a.den.is_one();
@@ -379,23 +400,29 @@ bigrational & bigrational::operator = (const bigrational & a)
 ** comparisons
 **/
 
-int operator == (const bigrational & a, const bigrational & b)
-{ return (a.compare(b) == 0); }
+bool operator == (const bigrational & a, const bigrational & b)
+{ if (a.compare(b) == 0) return true; 
+  return false; }
 
-int operator != (const bigrational & a, const bigrational & b)
-{ return (a.compare(b) != 0); }
+bool operator != (const bigrational & a, const bigrational & b)
+{ if (a.compare(b) != 0) return true; 
+  return false; }
 
-int operator > (const bigrational & a, const bigrational & b)
-{ return (a.compare(b) > 0); }
+bool operator > (const bigrational & a, const bigrational & b)
+{ if (a.compare(b) > 0) return true; 
+  return false; }
 
-int operator >= (const bigrational & a, const bigrational & b)
-{ return (a.compare(b) >= 0); }
+bool operator >= (const bigrational & a, const bigrational & b)
+{ if (a.compare(b) >= 0) return true;
+  return false; }
 
-int operator < (const bigrational & a, const bigrational & b)
-{ return (a.compare(b) < 0); }
+bool operator < (const bigrational & a, const bigrational & b)
+{ if (a.compare(b) < 0) return true; 
+  return false; }
 
-int operator <= (const bigrational & a, const bigrational & b)
-{ return (a.compare(b) <= 0); }
+bool operator <= (const bigrational & a, const bigrational & b)
+{ if (a.compare(b) <= 0) return true;
+  return false; }
 
 /**
 ** operator overloading
