@@ -318,19 +318,19 @@ crt::get_prime ( lidia_size_t index ) const
 }
 
 bool
-crt::test_mode ( char m )
+crt::test_mode ( char md )
  {
   debug_handler( "crt", "test_mode( char )" ) ;
 
   bool ok ;
 
-  if ( mode == m )
+  if ( mode == md )
     {
       ok = true ;
     }
   else if ( mode == INIT )
     {
-      mode = m ;
+      mode = md ;
       ok = true ;
     }
   else
@@ -811,7 +811,7 @@ crt::get_result( bigint & big )
 
    if ( used == m->num_primes )
      {
-       big = (bigint) (d + 0.5) ;                    //assignment bigint <- double
+       big.assign(d + 0.5) ;                    //assignment bigint <- double
        multiply ( big, big, m->minus_M_mod_p ) ;
        add      ( big, big, H ) ;
 
@@ -1093,7 +1093,7 @@ crt::get_result( bigmod & big )
 
    if ( used == m->num_primes )
      {
-       t = (bigint) (d + 0.5) ;                    //assignment bigint <- double
+       t.assign(d + 0.5) ;                    //assignment bigint <- double
        multiply ( t, t, m->minus_M_mod_p ) ;
        add      ( t, t, H ) ;
 
@@ -1144,7 +1144,7 @@ crt::get_result ( bigmod *& big_vec, lidia_size_t & l )
      {
        for ( i = 0 ; i < l ; i++ )
 	 {
-	   t = (bigint) (dv[i] + 0.5) ;                  //assignment bigint <- double
+	   t.assign(dv[i] + 0.5) ;                  //assignment bigint <- double
 	   multiply ( t, t, m->minus_M_mod_p ) ;
 	   add      ( t, t, Hv[i] ) ;
 	   
@@ -1200,7 +1200,7 @@ crt::get_result( base_vector<bigmod> & big_vec )
      {
        for ( i = 0 ; i < l ; i++ )
 	 {
-	   t = (bigint) (dptr[i] + 0.5) ;                    //  assignment bigint <- double
+	   t.assign(dptr[i] + 0.5) ;                    //  assignment bigint <- double
 	   multiply ( t, t, m->minus_M_mod_p ) ;
 	   add      ( t, t, Hptr[i] ) ;
 
