@@ -6,16 +6,21 @@
 #include <LiDIA/timer.h>
 #endif
 
+
+
+// Use portability-random now, MM, 21.11.96
+
 // Changed this, so that it compiles on HP-UX, Thomas P., 17.02.95
-#if defined(HAVE_RANDOM)
-#include <stdlib.h>
-#else
-#if !defined(__i386) || defined(SOLARIS)
-#include <LiDIA/random.h>
-#endif
+// #if defined(HAVE_RANDOM)
+// #include <stdlib.h>
+// #else
+// #if !defined(__i386) || defined(SOLARIS)
+// #include <LiDIA/random.h>
+// #endif
 #if defined(HAVE_MAC_DIRS) || defined(__MWERKS__)
-    #include <LiDIA:random.h>
-#endif
+#include <LiDIA:random.h>
+#else 
+#include <LiDIA/random.h>
 #endif
 
 #if defined(__GNUG__) && !defined(__mips) && !defined(SOLARIS)
@@ -318,7 +323,7 @@ ecm(lidia_size_t index, int ecm_restlength, int jobs_avail, int job_buffer[30][5
   bigint old_modul = bigmod::modulus();
   bigmod::set_modulus(N);
   
-  single_factor fact;
+  rf_single_factor fact;
   bigint factor, Q, R;
   factor.assign_one();
 
@@ -547,7 +552,7 @@ ecm(lidia_size_t index, int jobs_avail, int job_buffer[30][5], ecm_primes & prim
   bigint old_modul = bigmod::modulus();
   bigmod::set_modulus(N);
   
-  single_factor fact;
+  rf_single_factor fact;
   bigint factor, Q, R;
   factor.assign_one();
    
