@@ -5,7 +5,7 @@
 #endif
 
 
-single_factor & single_factor::operator = (const single_factor & x)
+rf_single_factor & rf_single_factor::operator = (const rf_single_factor & x)
 {	
   single_base = x.single_base;
   single_exponent = x.single_exponent;
@@ -13,31 +13,31 @@ single_factor & single_factor::operator = (const single_factor & x)
   return *this;
 }
 
-bool operator < (const single_factor & a, const single_factor & b)
+bool operator < (const rf_single_factor & a, const rf_single_factor & b)
 {
    return (a.single_base < b.single_base);
 }
 
-bool operator <= (const single_factor & a, const single_factor & b)
+bool operator <= (const rf_single_factor & a, const rf_single_factor & b)
 {
    return (a.single_base <= b.single_base);
 }
    
-bool operator == (const single_factor & a, const single_factor & b)
+bool operator == (const rf_single_factor & a, const rf_single_factor & b)
 {
    return (a.single_base == b.single_base);
 }
 
-void swap(single_factor & a, single_factor & b)
+void swap(rf_single_factor & a, rf_single_factor & b)
 {
-  single_factor c;
+  rf_single_factor c;
   
   c = a;
   a = b;
   b = c;
 }
 
-istream & operator >> (istream &in, single_factor &f)
+istream & operator >> (istream &in, rf_single_factor &f)
 {
    bigint hb;
    int he;
@@ -104,7 +104,7 @@ pretty_print (ostream &out)
  
 }
 
-ostream & operator << (ostream &out, const single_factor &f)
+ostream & operator << (ostream &out, const rf_single_factor &f)
 {
    out << "(";
    out << f.single_base;
@@ -401,6 +401,28 @@ if ( factors[0].single_base.is_one() )
 
 }
 
+void rational_factorization::
+convert(base_vector<bigint> & basis, base_vector<int> & exponent)
+{ 
+ lidia_size_t i,len;
+ 
+ if (basis.size()!=exponent.size())
+    lidia_error_handler("rational_factorization", "different sizes of basis and exponent vectors");
+ else
+   {len=basis.size();  
+    factors.set_size(len) ;
+    for (i=0; i<len; i++)
+     { factors[i].single_base = basis[i];
+       factors[i].single_exponent = exponent[i];
+       factors[i].factor_state = dont_know;
+     }
+     compose();  
+   }
+
+} 
+
+
+ 
 
 void rational_factorization::
 assign(const bigrational & n)
@@ -452,6 +474,20 @@ exponent(lidia_size_t index) const
   return factors[index].single_exponent;
 }
 
+void rational_factorization::
+set_exponent(lidia_size_t index, int expo)
+{
+  if ( (index < 0) || (index >= no_of_comp()) )
+    lidia_error_handler("rational_factorization","set_exponent::index out of range");
+  
+  factors[index].single_exponent = expo;
+  
+  if (expo == 0)
+    {
+      compose();
+    }
+}
+
 
 bool rational_factorization::
 is_prime_factor(lidia_size_t index) 
@@ -550,7 +586,7 @@ compose()
 }
 
 void rational_factorization::
-compose( sort_vector< single_factor > & v )
+compose( sort_vector< rf_single_factor > & v )
 {
   lidia_size_t i, j, len = v.size() - 1;  
 
@@ -697,9 +733,9 @@ void divide(rational_factorization & f,
 	    const rational_factorization & a, 
 	    const rational_factorization & b)
 {
-  f.assign(b);
-  f.invert();
-  f.factors.concat(a.factors, f.factors);
+  rational_factorization h(b);
+  h.invert();
+  f.factors.concat(a.factors, h.factors);
 
   if ( a.decomp_state == prime && b.decomp_state == prime )
     f.decomp_state = prime;
@@ -712,13 +748,13 @@ void divide(rational_factorization & f,
 
 
 void rational_factorization::
-refine2( sort_vector< single_factor > & v , single_factor & SF ,
-	 const single_factor & a, const single_factor & b )
+refine2( sort_vector< rf_single_factor > & v , rf_single_factor & SF ,
+	 const rf_single_factor & a, const rf_single_factor & b )
 
      // we assume vector v to be of expanding size !!!
 
      // compute refinement of {a,b} and store the
-     // last single_factor of this factorization individually
+     // last rf_single_factor of this factorization individually
      // in SF
      // thus {v[0],...,v[n],SF} represents the refinement of {a,b}
 {
@@ -726,7 +762,7 @@ refine2( sort_vector< single_factor > & v , single_factor & SF ,
   
   lidia_size_t i = 0, last_index = 2;
   int j;
-  single_factor sf;
+  rf_single_factor sf;
   bigint q, r;
   
   v.set_size ( 2 ) ;   
@@ -803,8 +839,8 @@ refine ( void )
 {
   debug_handler ( "rational_factorization" , "refine()" ) ;
   
-  sort_vector< single_factor > v(0,EXPAND), w(0,EXPAND) ;
-  single_factor b, c, sf ;
+  sort_vector< rf_single_factor > v(0,EXPAND), w(0,EXPAND) ;
+  rf_single_factor b, c, sf ;
   
   lidia_size_t i, j, k, l = factors.size();
 
@@ -866,7 +902,7 @@ refine ( const bigint & x )
   
   bigint old, n, res, d ;
   
-  single_factor sf ;
+  rf_single_factor sf ;
   
   j = len;
   
@@ -933,7 +969,7 @@ refine_comp (lidia_size_t index, const bigint &x)
   int e = 0;
   bigint old, n, res, d;
   
-  single_factor sf ;
+  rf_single_factor sf ;
   
   bool rc = false ;
   
