@@ -34,7 +34,7 @@
 // * Tr_lll_trans_var_dbl_gensys(T)
 //
 // Modified lll
-// * Tr_mlll_dbl(v)  (not yet implemented, possible ???)
+// * Tr_mlll_dbl(v)  (not yet implemented, possible ?)
 // 
 
 
@@ -1316,7 +1316,7 @@ void bigfloat_lattice_gensys::Tr_lll_var_dbl(math_matrix< bigint >& Bi, sdigit b
   cl(j);
   lidia_size_t k;
   sdigit old_prec;
-  sdigit bit_diff;
+  sdigit bit_diff=0;
   sdigit bi_bit_len;
   sdigit old_bit_prec;
   double bound=exp(log(2)*52/2);
@@ -1443,6 +1443,7 @@ void bigfloat_lattice_gensys::Tr_lll_var_dbl(math_matrix< bigint >& Bi, sdigit b
 //
 // end of orthogonalization
 //
+
         Fc=false;
         korr=false;
 //
@@ -1513,6 +1514,7 @@ void bigfloat_lattice_gensys::Tr_lll_var_dbl(math_matrix< bigint >& Bi, sdigit b
           }
       }
 
+
     debug_handler("bigfloat_lattice_gensys", "Tr_lll_var_dbl(Bi, Factor) [4]");
     if (y_par*vect[k-1] > my[k-1][k]*my[k-1][k]*vect[k-1]+vect[k])       
       {
