@@ -62,12 +62,12 @@ bigfloat_lattice_gensys::bigfloat_lattice_gensys():math_matrix< bigfloat > ()
   do_some_other_stuff();
 }
 
-bigfloat_lattice_gensys::bigfloat_lattice_gensys(lidia_size_t n):math_matrix< bigfloat > (n,1)
-{
-  debug_handler("bigfloat_lattice_gensys", "bigfloat_lattice_gensys(n)");
-  init_parameters();
-  do_some_other_stuff();
-}
+// bigfloat_lattice_gensys::bigfloat_lattice_gensys(lidia_size_t n):math_matrix< bigfloat > (n,1)
+// {
+//   debug_handler("bigfloat_lattice_gensys", "bigfloat_lattice_gensys(n)");
+//   init_parameters();
+//   do_some_other_stuff();
+// }
 
 bigfloat_lattice_gensys::bigfloat_lattice_gensys(lidia_size_t m, lidia_size_t n):math_matrix< bigfloat > (m,n)
 {
@@ -80,7 +80,7 @@ bigfloat_lattice_gensys::bigfloat_lattice_gensys(lidia_size_t m, lidia_size_t n)
 // Constructor with dimension (m, n) 
 // rows and columns and values bigfloat **abf
 // 
-bigfloat_lattice_gensys::bigfloat_lattice_gensys(lidia_size_t m, lidia_size_t n, bigfloat **abf):math_matrix< bigfloat > (m,n,abf)
+bigfloat_lattice_gensys::bigfloat_lattice_gensys(lidia_size_t m, lidia_size_t n, const bigfloat **abf):math_matrix< bigfloat > (m,n,abf)
 {
   debug_handler("bigfloat_lattice_gensys", "bigfloat_lattice_gensys(m, n, abf)");
   init_parameters();
