@@ -65,12 +65,12 @@ bigint_lattice_gensys::bigint_lattice_gensys():math_matrix< bigint > ()
   do_some_other_stuff();
 }
 
-bigint_lattice_gensys::bigint_lattice_gensys(lidia_size_t n):math_matrix< bigint > (n,1)
-{
-  debug_handler("bigint_lattice_gensys", "bigint_lattice_gensys(n)");
-  init_parameters();
-  do_some_other_stuff();
-}
+// bigint_lattice_gensys::bigint_lattice_gensys(lidia_size_t n):math_matrix< bigint > (n,1)
+// {
+//   debug_handler("bigint_lattice_gensys", "bigint_lattice_gensys(n)");
+//   init_parameters();
+//   do_some_other_stuff();
+// }
 
 bigint_lattice_gensys::bigint_lattice_gensys(lidia_size_t m, lidia_size_t n):math_matrix< bigint > (m,n)
 {
@@ -83,7 +83,7 @@ bigint_lattice_gensys::bigint_lattice_gensys(lidia_size_t m, lidia_size_t n):mat
 // Constructor with dimension (m, n) 
 // rows and columns and values bigint **abi
 // 
-bigint_lattice_gensys::bigint_lattice_gensys(lidia_size_t m, lidia_size_t n, bigint **abi):math_matrix< bigint > (m,n,abi)
+bigint_lattice_gensys::bigint_lattice_gensys(lidia_size_t m, lidia_size_t n, const bigint **abi):math_matrix< bigint > (m,n,abi)
 {
   debug_handler("bigint_lattice_gensys", "bigint_lattice_gensys(m, n, abi)");
   init_parameters();
