@@ -39,7 +39,7 @@ lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m)
 //
 // Initialising constructors
 //
-lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, double** ado)
+lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, const double** ado)
 {
   debug_handler("lattice_gensys","lattice_gensys(n, m, ado)");
   if ((n > 0) && (m > 0))
@@ -56,7 +56,7 @@ lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, double** ado)
                       "illegal lattice size !");
 }
 
-lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, bigint** abi)
+lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, const bigint** abi)
 {
   debug_handler("lattice_gensys","lattice_gensys(n, m, abi)");
   if ((n > 0) && (m > 0))
@@ -73,7 +73,7 @@ lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, bigint** abi)
                       "illegal lattice size !");
 }
 
-lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, bigfloat** abf)
+lattice_gensys::lattice_gensys(lidia_size_t n, lidia_size_t m, const bigfloat** abf)
 {
   debug_handler("lattice_gensys","lattice_gensys(n, m, abf)");
   if ((n > 0) && (m > 0))
@@ -319,7 +319,8 @@ void lattice_gensys::sto(lidia_size_t x, lidia_size_t y, const double& d)
   debug_handler("lattice_gensys","sto(x, y, d)"); 
   if (comp_mode == bigint_mode) 
     {
-      bigint temp(d);  
+      bigint temp; 
+      temp.assign(d);
       lbi->sto(x, y, temp);
     }
   else
@@ -518,7 +519,7 @@ void lattice_gensys::member(math_matrix< bigfloat >& Mbf)
     }
 }
 
-void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, double** ado)
+void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, const double** ado)
 {
   debug_handler("lattice_gensys","sto(n, m, ado)");
   cl(x);
@@ -555,7 +556,7 @@ void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, double** ado)
                       "illegal array size !");
 }
  
-void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, bigint** abi)
+void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, const bigint** abi)
 {
   debug_handler("lattice_gensys","sto(n, m, abi)");
   cl(x);
@@ -586,7 +587,7 @@ void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, bigint** abi)
                       "illegal array size !");
 }
 
-void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, bigfloat** abf)
+void lattice_gensys::sto(lidia_size_t n, lidia_size_t m, const bigfloat** abf)
 {
   debug_handler("lattice_gensys","sto(n, m, abf)");
   cl(x);
