@@ -1,11 +1,5 @@
 
 #if defined(HAVE_MAC_DIRS) || defined(__MWERKS__)
-#include <LiDIA:bigint.h>
-#include <LiDIA:bigmod.h>
-#include <LiDIA:bigrational.h>
-#include <LiDIA:bigfloat.h>
-#include <LiDIA:bigcomplex.h>
-
 #include <LiDIA:base_matrix.h>
 #include <LiDIA:base_matrix.c>
 
@@ -16,12 +10,6 @@
 
 #else
 
-#include <LiDIA/bigint.h> 
-#include <LiDIA/bigmod.h>
-#include <LiDIA/bigrational.h>
-#include <LiDIA/bigfloat.h>
-#include <LiDIA/bigcomplex.h> 
-
 #include <LiDIA/base_matrix.h>
 #include <LiDIA/base_matrix.c>
 
