@@ -73,11 +73,11 @@ main()
   dz << "testing constructor" << endl << flush;
   math_matrix < TYPE > A(3, 1);
   math_matrix < TYPE > B(2, 3);
-  math_matrix < TYPE > C(2, 3, wertearray);
+  math_matrix < TYPE > C(2, 3, (const TYPE **)wertearray);
   math_matrix < TYPE > D(C);
   math_matrix < TYPE > E = C;
   math_matrix < TYPE > F = C;
-  dz << A << B << C << D << E << F << flush;
+  // dz << A << B << C << D << E << F << flush;
   cout << ".............................completed" << endl << flush;
 
   A.read_from_stream(in);
