@@ -1,12 +1,4 @@
 #if defined(HAVE_MAC_DIRS) || defined(__MWERKS__)
-#include <LiDIA:bigint.h>
-#include <LiDIA:bigmod.h>
-#include <LiDIA:bigrational.h>
-#include <LiDIA:bigfloat.h>
-#include <LiDIA:bigcomplex.h>
-#include <LiDIA:rational_factorization.h>
-#include <LiDIA:coeff_sparse_power_serie.h>
-
 #include <LiDIA:base_vector.h>
 #include <LiDIA:base_vector.c>
 #include <LiDIA:file_vector.h>
@@ -17,14 +9,6 @@
 #include <LiDIA:sort_vector.c>
 #include <LiDIA:lidia_vector.h>
 #else
-#include <LiDIA/bigint.h>
-#include <LiDIA/bigmod.h>
-#include <LiDIA/bigrational.h>
-#include <LiDIA/bigfloat.h>
-#include <LiDIA/bigcomplex.h>
-#include <LiDIA/rational_factorization.h>
-#include <LiDIA/coeff_sparse_power_serie.h>
-
 #include <LiDIA/base_vector.h>
 #include <LiDIA/base_vector.c>
 #include <LiDIA/file_vector.h>
